import 'package:sport/bean/rank_game_info.dart';
import 'package:sport/bean/sport_target.dart';
import 'package:sport/services/Converter.dart';

import 'game.dart' as Game;
import 'user.dart';

class SportDetail {
  int durationTotalDay;
  int durationTarget;
  SportTarget target;
  NextAchievement nextAchievement;
  RecordsTodaySum recordsTodaySum;
  List<RecordsToday> recordsToday;
  List<GamesSum> gamesSum;
  List<Achievement> achievements;

  SportDetail(
      {this.durationTotalDay,
      this.durationTarget,
      this.target,
      this.nextAchievement,
      this.recordsTodaySum,
      this.recordsToday,
      this.gamesSum,
      this.achievements});

  SportDetail.fromJson(Map<String, dynamic> json) {
    durationTotalDay = json['duration_total_day'];
    durationTarget = json['duration_target'];
    target = json['target'] != null ? new SportTarget.fromJson(json['target']) : null;
    nextAchievement = json['next_achievement'] != null ? new NextAchievement.fromJson(json['next_achievement']) : null;
    recordsTodaySum = json['records_today_sum'] != null ? new RecordsTodaySum.fromJson(json['records_today_sum']) : null;
    if (json['records_today'] != null) {
      recordsToday = new List<RecordsToday>();
      json['records_today'].forEach((v) {
        recordsToday.add(new RecordsToday.fromJson(v));
      });
    }
    if (json['games_sum'] != null) {
      gamesSum = new List<GamesSum>();
      json['games_sum'].forEach((v) {
        gamesSum.add(new GamesSum.fromJson(v));
      });
    }
    if (json['achievements'] != null) {
      achievements = new List<Achievement>();
      json['achievements'].forEach((v) {
        achievements.add(new Achievement.fromJson(v));
      });
    }
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = new Map<String, dynamic>();
    data['duration_total_day'] = this.durationTotalDay;
    data['duration_target'] = this.durationTarget;
    if (this.target != null) {
      data['target'] = this.target.toJson();
    }
    if (this.nextAchievement != null) {
      data['next_achievement'] = this.nextAchievement.toJson();
    }
    if (this.recordsTodaySum != null) {
      data['records_today_sum'] = this.recordsTodaySum.toJson();
    }
    if (this.recordsToday != null) {
      data['records_today'] = this.recordsToday.map((v) => v.toJson()).toList();
    }
    if (this.gamesSum != null) {
      data['games_sum'] = this.gamesSum.map((v) => v.toJson()).toList();
    }
    if (this.achievements != null) {
      data['achievements'] = this.achievements.map((v) => v.toJson()).toList();
    }
    return data;
  }
}

class NextAchievement {
  Achievement achievement;
  int durationDiff;

  NextAchievement({this.achievement, this.durationDiff});

  NextAchievement.fromJson(Map<String, dynamic> json) {
    achievement = json['achievement'] != null ? new Achievement.fromJson(json['achievement']) : null;
    durationDiff = json['duration_diff'];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = new Map<String, dynamic>();
    if (this.achievement != null) {
      data['achievement'] = this.achievement.toJson();
    }
    data['duration_diff'] = this.durationDiff;
    return data;
  }
}

class RecordsTodaySum {
  int id;
  int userId;
  int duration;
  String createdAt;
  int consume;
  int jumpCount;
  int crouchCount;
  int screen ;
  double score;

  RecordsTodaySum({this.id, this.userId, this.duration, this.createdAt, this.consume, this.jumpCount, this.crouchCount, this.screen, this.score});

  RecordsTodaySum.fromJson(Map<String, dynamic> json) {
    id = json['id'];
    userId = json['user_id'];
    duration = json['duration'];
    createdAt = json['created_at'];
    consume = json['consume'];
    jumpCount = json['jump_count'];
    crouchCount = json['crouch_count'];
    screen = json['screen'];
    score = Converter.toDouble(json['score']);
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = new Map<String, dynamic>();
    data['id'] = this.id;
    data['user_id'] = this.userId;
    data['duration'] = this.duration;
    data['created_at'] = this.createdAt;
    data['consume'] = this.consume;
    data['jump_count'] = this.jumpCount;
    data['crouch_count'] = this.crouchCount;
    data['screen'] = this.screen;
    return data;
  }
}

class RecordsToday {
  int id;
  int userId;
  int duration;
  String createdAt;
  int month;
  int consume;
  int gameId;
  int jumpCount;
  int crouchCount;
  double score;

  RecordsToday({this.id, this.userId, this.duration, this.createdAt,this.month, this.consume, this.gameId, this.jumpCount, this.crouchCount, this.score});

  RecordsToday.fromJson(Map<String, dynamic> json) {
    id = json['id'];
    userId = json['user_id'];
    duration = json['duration'];
    createdAt = json['created_at'];
    month = json['month'];
    consume = json['consume'];
    gameId = json['game_id'];
    jumpCount = json['jump_count'];
    crouchCount = json['crouch_count'];
    score = Converter.toDouble(json['score']);
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = new Map<String, dynamic>();
    data['id'] = this.id;
    data['user_id'] = this.userId;
    data['duration'] = this.duration;
    data['created_at'] = this.createdAt;
    data['consume'] = this.consume;
    data['game_id'] = this.gameId;
    data['jump_count'] = this.jumpCount;
    data['crouch_count'] = this.crouchCount;
    data['score'] = this.score;
    return data;
  }
}

class GamesSum {
  int id;
  int userId;
  int gameId;
  String lastPlayAt;
  int playCount;
  int durationTotal;
  double scoreTotal;
  int consumeTotal;
  double scoreMax;
  Game.GameInfoData game;

  GamesSum(
      {this.id, this.userId, this.gameId, this.lastPlayAt, this.playCount, this.durationTotal, this.scoreTotal, this.consumeTotal, this.scoreMax, this.game});

  GamesSum.fromJson(Map<String, dynamic> json) {
    id = json['id'];
    userId = json['user_id'];
    gameId = json['game_id'];
    lastPlayAt = json['last_play_at'];
    playCount = json['play_count'];
    durationTotal = json['duration_total'];
    scoreTotal = Converter.toDouble(json['score_total']);
    consumeTotal = json['consume_total'];
    scoreMax = Converter.toDouble(json['score_max']);
    game = json['game'] != null ? new Game.GameInfoData.fromJson(json['game']) : null;
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = new Map<String, dynamic>();
    data['id'] = this.id;
    data['user_id'] = this.userId;
    data['game_id'] = this.gameId;
    data['last_play_at'] = this.lastPlayAt;
    data['play_count'] = this.playCount;
    data['duration_total'] = this.durationTotal;
    data['score_total'] = this.scoreTotal;
    data['consume_total'] = this.consumeTotal;
    data['score_max'] = this.scoreMax;
    data['game'] = this.game;
    return data;
  }
}

class SportDetailSimple {
  RecordsTodaySum sum;
  List<RecordsToday> records;

  SportDetailSimple({this.sum, this.records});

  SportDetailSimple.fromJson(Map<String, dynamic> json) {
    sum = json['sum'] != null ? new RecordsTodaySum.fromJson(json['sum']) : null;
    if (json['records'] != null) {
      records = new List<RecordsToday>();
      json['records'].forEach((v) {
        records.add(new RecordsToday.fromJson(v));
      });
    }
  }
}
