import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:sport/bean/feed_back.dart';
import 'package:sport/bean/game.dart';
import 'package:sport/pages/home/sport_detail_page.dart';
import 'package:sport/pages/my/feedback_detail_page.dart';
import 'package:sport/pages/my/feedback_page.dart';
import 'package:sport/router/navigator_util.dart';
import 'package:sport/router/routes.dart';
import 'package:sport/services/api/inject_api.dart';
import 'package:sport/sharesdk/wechat.dart';
import 'package:sport/utils/toast.dart';
import 'package:sport/widgets/dialog/request_dialog.dart';

//import 'package:sharesdk_plugin/sharesdk_defines.dart';
//import 'package:sharesdk_plugin/sharesdk_interface.dart';
//import 'package:sharesdk_plugin/sharesdk_map.dart';
import 'package:video_player/video_player.dart';

class GameDetailsVideo extends StatefulWidget {
  final Function changeIsFullScreen;
  final bool _isFullScreen;
  VideoPlayerController controller;
  Duration timeEnd;
  Duration timeStart;
  bool isLoading;
  GameInfoData data;

  GameDetailsVideo(this.changeIsFullScreen, this._isFullScreen, this.controller, this.timeStart, this.timeEnd, this.isLoading, this.data, {Key key})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    // TODO: implement createState
    return _GameDetailsVideoState();
  }
}

class _GameDetailsVideoState extends State<GameDetailsVideo> with WechatMixin, InjectLoginApi {
  void changeIsFullScreen;
  Timer timer;
  var opacity = 1.0;
  final spinkit = SpinKitDualRing(
    color: Colors.white,
  );

  @override
  void initState() {
    super.initState();
  }

  void initdelay() {
    if (timer != null) {
      timer.cancel();
    }
    setState(() {
      opacity = 1;
    });
    timer = Timer(new Duration(seconds: 3), () {
      setState(() {
        if (mounted) opacity = 0;
      });
    });
  }

  @override
  void dispose() {
    super.dispose();
    timer?.cancel();
  }

  @override
  Widget build(BuildContext context) {
    final size = MediaQuery.of(context).size;
    final width = size.width;
    final height = size.height;
    bool isLoading = widget.isLoading;
    bool isPlaying = widget.controller.value.isPlaying;
    Duration timeStart = widget.timeStart;
    Duration timeEnd = widget.timeEnd;
    VideoPlayerController controller = widget.controller;
    return Container(
        color: Colors.black,
//      约束宽高的
        child: AspectRatio(
          aspectRatio: 1.78,
          child: Stack(
            children: <Widget>[
              isLoading == false
                  ? InkWell(
                      child: (VideoPlayer(controller)),
                      onTap: () {
                        initdelay();
                      },
                      onDoubleTap: () {
                        if (isPlaying) {
                          controller.pause();
                        } else {
                          controller.play();
                        }
                      },
                    )
                  : spinkit,
              isPlaying != true && isLoading == false
                  ? InkWell(
                      child: Center(
                          child: Image.asset(
                        "lib/assets/img/game_icon_play.png",
                        width: 50,
                        height: 50,
                      )),
                      onTap: () async {
                        if (isPlaying) {
                          controller.pause();
                        } else {
                          if (controller.value.position.compareTo(controller.value.duration) == 0) {
                            controller.seekTo(Duration.zero);
                          } else {
                            initdelay();
                          }
                          controller.play();
                        }
                      },
                    )
                  : Container(),
              Positioned(
                left: 0,
                right: 0,
                child: AnimatedOpacity(
                  duration: new Duration(seconds: 1),
                  opacity: opacity,
                  child: Container(
                    padding: EdgeInsets.fromLTRB(10.0,0,10.0,10.0),
                    decoration: BoxDecoration(
                        gradient: LinearGradient(begin: Alignment.topCenter, end: Alignment.bottomCenter, colors: [
                      Color(0xFF000000).withOpacity(.5),
                      Color(0x00000000)
//                Color(0x00000000),
                    ])),
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      children: <Widget>[
                        InkWell(
                          child: Image.asset(
                            "lib/assets/img/topbar_return_white.png",
                            width: 26,
                            height: 38,
                          ),
                          onTap: () {
                            if (widget._isFullScreen) {
                              widget.changeIsFullScreen();
                            } else {
                              // 原生的返回上一页?
                              Navigator.pop(context);
                            }
                          },
                        ),
                        PopupMenuButton<String>(
                          icon: Image.asset(
                            "lib/assets/img/topbar_more_white.png",
                            width: 26,
                            height: 38,
                          ),
                          onSelected: (val) async {
                            switch (val) {
                              case '运动记录':
                                NavigatorUtil.goGameHistory(context, widget.data);
                                break;
                              case '设备测试':
                                ToastUtil.show("设备测试");
                                break;
                              case '问题反馈':
                                FeedTypeInfoData group;
                                await request(context, () async {
                                  FeedTypeInfo _feedTypeInfo = await loginApi.getFeedBackTypes();
                                  if (_feedTypeInfo.code == 0) {
                                    group = _feedTypeInfo.data.singleWhere((element) => element.groupId == '3');
                                  }
                                });
                                if (group != null) {
                                  NavigatorUtil.goPage(context, (context) => FeedbackDetailPage(group));
                                }
                                break;
                              case '清除数据':
                                NavigatorUtil.go(context, Routes.gamelistpage);
                                break;
                            }
                          },
                          itemBuilder: (context) {
                            return <PopupMenuEntry<String>>[
                              PopupMenuItem<String>(
                                value: '运动记录',
                                child: Center(
                                  child: Text(
                                    '运动记录',
                                  ),
                                ),
                              ),
                              PopupMenuItem<String>(
                                value: '设备测试',
                                child: Center(
                                    child: Text(
                                  '设备测试',
                                )),
                              ),
                              PopupMenuItem<String>(
                                value: '问题反馈',
                                child: Center(
                                    child: Text(
                                  '问题反馈',
                                )),
                              ),
                              PopupMenuItem<String>(
                                value: '清除数据',
                                child: Center(
                                    child: Text(
                                  '清除数据',
                                )),
                              )
                            ];
                          },
                        )
                      ],
                    ),
                  ),
                ),
              ),
              Positioned(
                bottom: 0,
                left: 0,
                right: 0,
                child: AnimatedOpacity(
                  duration: new Duration(seconds: 1),
                  opacity: opacity,
                  child: Container(
                    padding: EdgeInsets.symmetric(horizontal: 17.0),
                    decoration: BoxDecoration(
                        gradient: LinearGradient(begin: Alignment.topCenter, end: Alignment.bottomCenter, colors: [
                      Color(0x00000000), Color(0xFF000000).withOpacity(.5)
//                Color(0x00000000),
                    ])),
                    child: Column(
                      mainAxisSize: MainAxisSize.min,
                      children: <Widget>[
                        Row(
                          mainAxisAlignment: MainAxisAlignment.end,
                          children: <Widget>[
                            InkWell(
                              child: isPlaying != true
                                  ? Image.asset(
                                      "lib/assets/img/gamevideo_play_white.png",
                                      width: 26,
                                      height: 38,
                                    )
                                  : Image.asset(
                                      "lib/assets/img/gamevideo_suspend.png",
                                      width: 26,
                                      height: 38,
                                    ),
                              onTap: () {
                                if (isPlaying) {
                                  controller.pause();
                                } else {
                                  if (controller.value.position.compareTo(controller.value.duration) == 0) {
                                    controller.seekTo(Duration.zero);
                                  } else {
                                    initdelay();
                                  }
                                  controller.play();
                                }
                              },
                            ),
                            Padding(
                              padding: EdgeInsets.symmetric(horizontal: 5.0),
                              child: timeStart != null
                                  ? Text(timeStart.toString().substring(2, 7), style: TextStyle(color: Colors.white))
                                  : Text("00:00", style: TextStyle(color: Colors.white)),
                            ),
                            Expanded(
                                child: VideoProgressIndicator(controller,
                                    allowScrubbing: true, // 是否可以拖动
                                    padding: const EdgeInsets.all(0),
                                    colors: VideoProgressColors(playedColor: Theme.of(context).accentColor))),
                            Padding(
                              padding: EdgeInsets.symmetric(horizontal: 5.0),
                              child: timeEnd != null
                                  ? Text(timeEnd.toString().substring(2, 7), style: TextStyle(color: Colors.white))
                                  : Text("00:00", style: TextStyle(color: Colors.white)),
                            ),
                            InkWell(
                                // 全屏按钮
                                child: Image.asset(
                                  "lib/assets/img/gamevideo_fullscreen.png",
                                  width: 26,
                                  height: 38,
                                ),
                                onTap: () {
                                  widget.changeIsFullScreen();
                                })
                          ],
                        ),
                      ],
                    ),
                  ),
                ),
              ),
//            ClosedCaption(text: _controller.value.caption.text),
            ],
          ),
        ));
  }
}
