import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:sport/bean/rank_game_info.dart';
import 'package:sport/bean/user.dart';
import 'package:sport/widgets/appbar.dart';
import 'package:sport/widgets/misc.dart';
import 'package:sport/widgets/space.dart';

class RankPeopleDetailsPage extends StatefulWidget {
  final User _data;

  RankPeopleDetailsPage(this._data);

  @override
  State<StatefulWidget> createState() {
    // TODO: implement createState
    return _RankPeopleDetailsPageState();
  }
}

class _RankPeopleDetailsPageState extends State<RankPeopleDetailsPage> {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    final data = widget._data;
    return Scaffold(
        backgroundColor: Colors.white,
        appBar: AppBar(
          backgroundColor: Colors.white,
          elevation: 0,
          leading: buildBackButton(context),
        ),
        body: SingleChildScrollView(
//        physics: ClampingScrollPhysics(),
          child: Container(
            color: Colors.white,
            width: double.infinity,
            child: Column(
              children: <Widget>[
                Container(
                  width: 90.0,
                  height: 90.0,
                  alignment: Alignment.center,
                  child: ClipRRect(
                    borderRadius: BorderRadius.all(Radius.circular(50.0)),
                    child: CachedNetworkImage(
                      imageUrl: data.userAvatar,
                      errorWidget: (context, _, __) => Container(
                        color: Colors.grey,
                      ),
                      width: 100,
                      height: 100,
                      fit: BoxFit.cover,
                    ),
                  ),
                ),
                Padding(
                  padding: EdgeInsets.symmetric(vertical: 12.0),
                  child: Text(
                    "${data.userName}",
                    style: TextStyle(color: Colors.black, fontSize: 16.0, fontWeight: FontWeight.bold),
                  ),
                ),
                Padding(
                  padding: EdgeInsets.only(bottom: 36.0),
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: <Widget>[
                      data.userGender != 0
                          ? Text(
                              "${data.userGender == 1 ? '男' : '女'}${data.userAge == null ? '' : '  ${data.userAge}岁'}",
                              style: TextStyle(),
                              textAlign: TextAlign.center,
                            )
                          : Text("")
                    ],
                  ),
                ),
                Divider(
                  endIndent: 12,
                  indent: 12,
                ),
                Container(
                  padding: EdgeInsets.symmetric(horizontal: 20.0, vertical: 21.0),
                  child: Column(
                    children: <Widget>[
                      Padding(
                        padding: EdgeInsets.only(
                          bottom: 17.0,
                        ),
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: <Widget>[
                            Text(
                              "榜单排名",
                              style: TextStyle(color: Color.fromRGBO(51, 51, 51, 1)),
                            ),
                            Text(
                              "第 ${data.position} 名",
                              style: TextStyle(color: Color.fromRGBO(255, 196, 0, 1)),
                            )
                          ],
                        ),
                      ),
                      if(data.score != null)
                      Padding(
                        padding: EdgeInsets.only(
                          bottom: 17.0,
                        ),
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: <Widget>[
                            Text(
                              "通关评分",
                              style: TextStyle(color: Color.fromRGBO(51, 51, 51, 1)),
                            ),
                            Text(
                              "${data.score.toStringAsFixed(1)}分",
                              style: TextStyle(color: Color.fromRGBO(153, 153, 153, 1)),
                            )
                          ],
                        ),
                      ),
                      Padding(
                        padding: EdgeInsets.only(
                          bottom: 17.0,
                        ),
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: <Widget>[
                            Text(
                              "通关日期",
                              style: TextStyle(color: Color.fromRGBO(51, 51, 51, 1)),
                            ),
                            Text(
                              "${data.finishAt}",
                              style: TextStyle(color: Color.fromRGBO(153, 153, 153, 1)),
                            )
                          ],
                        ),
                      ),
                      Padding(
                        padding: EdgeInsets.only(
                          bottom: 17.0,
                        ),
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: <Widget>[
                            Text(
                              "通关时长",
                              style: TextStyle(color: Color.fromRGBO(51, 51, 51, 1)),
                            ),
                            Text(
                              "${data.duration}分钟",
                              style: TextStyle(color: Color.fromRGBO(153, 153, 153, 1)),
                            )
                          ],
                        ),
                      ),
                      Padding(
                        padding: EdgeInsets.only(
                          bottom: 17.0,
                        ),
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: <Widget>[
                            Text(
                              "消耗能量",
                              style: TextStyle(color: Color.fromRGBO(51, 51, 51, 1)),
                            ),
                            Text(
                              "${data.consume} kal",
                              style: TextStyle(color: Color.fromRGBO(153, 153, 153, 1)),
                            )
                          ],
                        ),
                      ),
                      Padding(
                        padding: EdgeInsets.only(
                          bottom: 17.0,
                        ),
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: <Widget>[
                            Text(
                              "运动效率",
                              style: TextStyle(color: Color.fromRGBO(51, 51, 51, 1)),
                            ),
                            Text(
                              "${(data.consume / data.duration).toStringAsFixed(2)} kal/min",
                              style: TextStyle(color: Color.fromRGBO(153, 153, 153, 1)),
                            )
                          ],
                        ),
                      ),
                    ],
                  ),
                ),
                Divider(
                  endIndent: 12,
                  indent: 12,
                ),
                Column(
                  children: <Widget>[
                    Space(
                      height: 10,
                    ),
                    // head
                    Stack(
                      alignment: Alignment.center,
                      children: <Widget>[
                        Image.asset(
                          "lib/assets/img/mine_image_achievement.png",
                          fit: BoxFit.fitWidth,
                          width: 240,
                        ),
                        Text(
                          "我的成就",
                          style: Theme.of(context).textTheme.subtitle1,
                        )
                      ],
                    ),
                    // bottom
                    Padding(
                      padding: EdgeInsets.all(12.0),
                      child: achievementGroupWidget(data?.achievements)
                    )
                  ],
                )
              ],
            ),
          ),
        ));
  }
}
