import 'dart:math';
import 'dart:ui';

import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/material.dart';
import 'package:flutter/painting.dart';
import 'package:sport/bean/feed_back.dart';
import 'package:sport/bean/game.dart';
import 'package:sport/bean/sport_detail.dart';
import 'package:sport/bean/sport_target.dart';
import 'package:sport/bean/sport_target_today.dart';
import 'package:sport/pages/home/target_modify.dart';
import 'package:sport/pages/my/feedback_detail_page.dart';
import 'package:sport/router/navigator_util.dart';
import 'package:sport/router/routes.dart';
import 'package:sport/services/api/inject_api.dart';
import 'package:sport/services/api/resp.dart';
import 'package:sport/utils/toast.dart';
import 'package:sport/widgets/appbar.dart';
import 'package:sport/widgets/chart.dart';
import 'package:sport/widgets/configurable_expansion_tile.dart';
import 'package:sport/widgets/decoration.dart';
import 'package:sport/widgets/dialog/alert_dialog.dart';
import 'package:sport/widgets/dialog/request_dialog.dart';
import 'package:sport/widgets/image.dart';
import 'package:sport/widgets/label.dart';
import 'package:sport/widgets/loading.dart';
import 'package:sport/widgets/misc.dart';
import 'package:sport/widgets/space.dart';

class SportDetailPage extends StatefulWidget {
  @override
  State<StatefulWidget> createState() => _PageState();
}

class _PageState extends State<SportDetailPage> with TickerProviderStateMixin, InjectApi, InjectLoginApi {
  final List<String> _TABS = ["日", "周", "月", "年"];
  ValueNotifier<String> _tab = ValueNotifier<String>("日");
  ValueNotifier<SportTargetToday> _valueNotifierTarget = ValueNotifier(null);
  ValueNotifier<SportDetail> _valueNotifierSportDetail = ValueNotifier(null);
  ValueNotifier<DateTime> _valueNotifierDate = ValueNotifier(DateTime.now());
  int _durationTarget = 60;
  SportDetail _sportDetail;

  @override
  void initState() {
    super.initState();
    api.getSportDetail().then((value) {
      _sportDetail = value.data;
      if (_sportDetail == null) return;
      _durationTarget = value.data.durationTarget;
      _valueNotifierSportDetail.value = _sportDetail;
      _valueNotifierTarget.value = SportTargetToday(remain: value.data.durationTarget - value.data.durationTotalDay, target: value.data.target);
      if (mounted) setState(() {});
      if (value.data.target == null) {
        _showFirstTarget();
      }
    });
  }

  @override
  void dispose() {
    super.dispose();
    _tab?.dispose();
    _valueNotifierTarget?.dispose();
    _valueNotifierSportDetail?.dispose();
    _valueNotifierDate?.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: CustomScrollView(
        slivers: <Widget>[
          buildSliverAppBar(context, "运动详情", backgroundColor: Theme.of(context).scaffoldBackgroundColor, actions: <Widget>[
            PopupMenuButton(
              icon: Image.asset("lib/assets/img/topbar_more.png"),
              onSelected: (action) async {
                switch (action) {
                  case "modify":
                    NavigatorUtil.go(context, Routes.targetModify).then((value) {
                      if (value == null) return;
                      if (value is SportTargetToday) {
                        _valueNotifierTarget.value = value;
                      }
                    });
                    break;
                  case "data":
                    FeedTypeInfoData group;
                    await request(context, () async {
                      FeedTypeInfo _feedTypeInfo = await loginApi.getFeedBackTypes();
                      if (_feedTypeInfo.code == 0) {
                        group = _feedTypeInfo.data.singleWhere((element) => element.groupId == '2');
                      }
                    });
                    if (group != null) {
                      NavigatorUtil.goPage(context, (context) => FeedbackDetailPage(group));
                    }
                    break;
                  case "share":
                    ToastUtil.show("分享什么东西~");
                    break;
                }
              },
              itemBuilder: (_) => [
                PopupMenuItem(
                  value: "modify",
                  child: Text("修改运动目标"),
                ),
                PopupMenuItem(
                  value: "data",
                  child: Text("数据报错"),
                ),
                PopupMenuItem(
                  value: "share",
                  child: Text("分享"),
                )
              ],
            )
          ]),
          SliverToBoxAdapter(
            child: _buildTargetWidget(),
          ),
          SliverToBoxAdapter(
            child: LabelWidget(title: "数据统计"),
          ),
          SliverToBoxAdapter(
            child: _buildDataWidget(),
          ),
        ],
      ),
    );
  }

  Widget _buildTargetWidget() {
    return ValueListenableBuilder<SportTargetToday>(
      valueListenable: _valueNotifierTarget,
      builder: (_, _value, ___) => Column(
        children: <Widget>[
          LabelWidget(title: "今日目标"),
          Container(
            margin: EdgeInsets.fromLTRB(12.0, 6.0, 12.0, 18.0),
            padding: EdgeInsets.fromLTRB(12.0, 12.0, 12.0, 12.0),
            decoration: circular(),
            child: _value == null
                ? RequestLoadingWidget()
                : Column(
                    children: <Widget>[
                      if (_value.target != null)
                        Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          children: <Widget>[
                            Text(_value.remain <= 0 ? "今日已完成" : "还差：${_value.remain} min", style: Theme.of(context).textTheme.subtitle1),
                            Text("强度：${_value.target.intension}", style: Theme.of(context).textTheme.subtitle1)
                          ],
                        ),
                      if (_value.target == null)
                        InkWell(
                          onTap: () {
                            NavigatorUtil.go(context, Routes.targetModify).then((value) {
                              if (value == null) return;
                              if (value is SportTargetToday) {
                                _valueNotifierTarget.value = value;
                              }
                            });
                          },
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.end,
                            children: <Widget>[
                              Text("设立目标", style: Theme.of(context).textTheme.subtitle1.copyWith(color: Theme.of(context).accentColor)),
                              Space(
                                width: 5,
                              ),
                              arrowRight6()
                            ],
                          ),
                        ),
                      Space(
                        height: 10,
                      ),
                      Container(child: LayoutBuilder(builder: (context, box) {
                        int minute = _sportDetail.durationTotalDay;
                        double percent = minute / (_value.target == null ? _durationTarget : _value.target.duration);
                        double w = 70;
                        double padding = box.maxWidth * percent - w / 2;
                        if (padding < w / 2) {
                          padding = 0;
                        }
                        if (padding > box.maxWidth - w) {
                          padding = box.maxWidth - w;
                        }
                        return Column(
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: <Widget>[
                            Padding(
                              padding: EdgeInsets.only(left: padding),
                              child: Container(
                                width: w,
                                alignment: padding == 0 ? Alignment.centerLeft : Alignment.center,
                                child: RichText(
                                  text: TextSpan(style: Theme.of(context).textTheme.subtitle2, children: <InlineSpan>[
                                    TextSpan(
                                      text: '$minute',
                                      style: Theme.of(context).textTheme.subtitle2.copyWith(fontWeight: FontWeight.bold, fontSize: 25),
                                    ),
                                    TextSpan(text: '分钟'),
                                  ]),
                                ),
                              ),
                            ),
                            CustomPaint(
                              painter: _ProgressBar(percent),
                              child: Container(
                                height: 21,
                              ),
                            ),
                          ],
                        );
                      })),
                      Space(
                        height: 2,
                      ),
                      Row(
                        mainAxisAlignment: MainAxisAlignment.spaceBetween,
                        children: <Widget>[
                          Text(
                            "0分钟",
                            style: Theme.of(context).textTheme.subtitle2,
                          ),
                          Text(
                            _value.target == null ? "未设立" : "${_value.target.duration}分钟",
                            style: Theme.of(context).textTheme.subtitle2,
                          )
                        ],
                      ),
                      Space(
                        height: 20,
                      ),
                      if (_sportDetail?.nextAchievement != null)
                        InkWell(
                          onTap: () => NavigatorUtil.goAchievementDetails(context, id: _sportDetail.nextAchievement.achievement.id),
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.spaceBetween,
                            children: <Widget>[
                              RichText(
                                  text: TextSpan(style: Theme.of(context).textTheme.bodyText2, children: <InlineSpan>[
                                TextSpan(text: '距离'),
                                TextSpan(
                                  text: ' ${_sportDetail.nextAchievement.achievement.name} ',
                                  style: Theme.of(context).textTheme.bodyText2.copyWith(color: Theme.of(context).accentColor),
                                ),
                                TextSpan(text: '还需运动 ${_sportDetail.nextAchievement.durationDiff} min'),
                              ])),
                              arrowRight4()
                            ],
                          ),
                        ),
                    ],
                  ),
          ),
        ],
      ),
    );
  }

  Widget _buildDataWidget() {
    return Container(
      margin: EdgeInsets.fromLTRB(12.0, 6.0, 12.0, 18.0),
      padding: EdgeInsets.fromLTRB(12.0, 12.0, 12.0, 12.0),
      decoration: circular(),
      child: Column(
        children: <Widget>[
          ValueListenableBuilder(
            valueListenable: _tab,
            builder: (BuildContext context, String value, Widget child) {
              return Column(
                children: <Widget>[
                  Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    mainAxisSize: MainAxisSize.min,
                    children: ["日", "/", "周", "/", "月", "/", "年"]
                        .map((e) => e == "/"
                            ? Container(
                                margin: EdgeInsets.symmetric(horizontal: 6.0),
                                color: Color(0xffdcdcdc),
                                width: 0.5,
                                height: 14,
                                transform: Matrix4.rotationZ(0.35),
                              )
                            : InkWell(
                                onTap: () {
                                  if (_valueNotifierSportDetail.value == null) return;
                                  _tab.value = e;
                                  _valueNotifierDate.value = DateTime.now();
                                  changeTab();
                                },
                                child: Container(
                                  margin: EdgeInsets.symmetric(horizontal: 12.0),
                                  padding: EdgeInsets.all(8.0),
                                  decoration: value == e ? BoxDecoration(color: Theme.of(context).accentColor, shape: BoxShape.circle) : null,
                                  child: Text(
                                    "$e",
                                    style: Theme.of(context).textTheme.subtitle1.copyWith(color: value == e ? Colors.white : Theme.of(context).accentColor),
                                  ),
                                )))
                        .toList(),
                  ),
                  Padding(
                    padding: const EdgeInsets.symmetric(horizontal: 30, vertical: 12),
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.center,
                      children: <Widget>[
                        GestureDetector(
                          behavior: HitTestBehavior.opaque,
                          onTap: () {
                            rollDate(-1);
                          },
                          child: Padding(
                            padding: const EdgeInsets.all(16.0),
                            child: arrowLeft(),
                          ),
                        ),
                        Expanded(
                          child: Center(
                            child: ValueListenableBuilder<DateTime>(
                                valueListenable: _valueNotifierDate,
                                builder: (_, time, ___) {
                                  int type = toType();
                                  if (type == 0) {
                                    return Text(
                                        "${time.year}.${'${time.month}'.padLeft(2, '0')}.${'${time.day}'.padLeft(2, '0')}  ${WEEK[time.weekday == 0 ? 6 : time.weekday - 1]}");
                                  } else if (type == 1) {
                                    DateTime start = DateTime(time.year, time.month, time.day - time.weekday + 1);
                                    DateTime end = DateTime(time.year, time.month, time.day + 6 - time.weekday + 1);
                                    print("$time ${time.weekday} ==  $start $end");
                                    return Text(
                                        "${start.year}.${'${start.month}'.padLeft(2, '0')}.${'${start.day}'.padLeft(2, '0')}  至  ${end.year}.${'${end.month}'.padLeft(2, '0')}.${'${end.day}'.padLeft(2, '0')}");
                                  } else if (type == 2) {
                                    return Text("${time.year}年${'${time.month}'.padLeft(2, '0')}月");
                                  } else if (type == 3) {
                                    return Text("${time.year}年");
                                  }
                                  return Text(" ");
                                }),
                          ),
                        ),
                        GestureDetector(
                          behavior: HitTestBehavior.opaque,
                          onTap: () {
                            rollDate(1);
                          },
                          child: Padding(
                            padding: const EdgeInsets.all(16.0),
                            child: arrowRight(),
                          ),
                        ),
                      ],
                    ),
                  ),
                ],
              );
            },
          ),
          Space(
            height: 12,
          ),
          ValueListenableBuilder<SportDetail>(
            valueListenable: _valueNotifierSportDetail,
            builder: (_, _value, ___) => _value == null
                ? Container(
                    height: 160,
                    child: RequestLoadingWidget(),
                  )
                : CustomPaint(
                    painter: Chart(_TABS.indexOf(_tab.value), _value.recordsToday, _valueNotifierDate.value)..initData(),
                    child: Container(
                      height: 160,
                    ),
                  ),
          ),
          Space(
            height: 6,
          ),
          ValueListenableBuilder<SportDetail>(
            valueListenable: _valueNotifierSportDetail,
            builder: (_, _value, ___) => (_value != null && _value.recordsTodaySum == null)
                ? Container()
                : Column(
                    children: <Widget>[
                      Row(
                        mainAxisAlignment: MainAxisAlignment.spaceBetween,
                        children: <Widget>[
                          Text(
                            "运动消耗",
                            style: Theme.of(context).textTheme.subtitle1,
                          ),
                          Text("${_value == null ? ".." : "${_value.recordsTodaySum.consume}"} kal", style: Theme.of(context).textTheme.subtitle1)
                        ],
                      ),
                      Space(
                        height: 8,
                      ),
                      Row(
                        mainAxisAlignment: MainAxisAlignment.spaceBetween,
                        children: <Widget>[
                          Text(
                            "运动时间",
                            style: Theme.of(context).textTheme.subtitle1,
                          ),
                          Text("${_value == null ? ".." : "${_value.recordsTodaySum.duration}"} 分钟", style: Theme.of(context).textTheme.subtitle1)
                        ],
                      ),
                      Space(
                        height: 8,
                      ),
                      Row(
                        mainAxisAlignment: MainAxisAlignment.spaceBetween,
                        children: <Widget>[
                          Text(
                            "跳跃次数",
                            style: Theme.of(context).textTheme.subtitle1,
                          ),
                          Text("${_value == null ? ".." : "${_value.recordsTodaySum.jumpCount}"} 次", style: Theme.of(context).textTheme.subtitle1)
                        ],
                      ),
                      Space(
                        height: 8,
                      ),
                      Row(
                        mainAxisAlignment: MainAxisAlignment.spaceBetween,
                        children: <Widget>[
                          Text(
                            "下蹲次数",
                            style: Theme.of(context).textTheme.subtitle1,
                          ),
                          Text("${_value == null ? ".." : "${_value.recordsTodaySum.crouchCount}"} 次", style: Theme.of(context).textTheme.subtitle1)
                        ],
                      ),

                      Row(
                        children: <Widget>[
                          Expanded(child: Divider()),
                          Padding(
                            padding: const EdgeInsets.all(16.0),
                            child: Text(
                              "相当于",
                              style: Theme.of(context).textTheme.subtitle1,
                            ),
                          ),
                          Expanded(child: Divider())
                        ],
                      ),
                      sportBeEquivalentTo(context, _value == null || _value.recordsTodaySum == null ? 0 : _value.recordsTodaySum.consume),
                    ],
                  ),
          ),
          Space(
            height: 10,
          ),
          Divider(),
          if (_sportDetail != null && _sportDetail.gamesSum != null && _sportDetail.gamesSum.isNotEmpty)
            ListView.separated(
                padding: EdgeInsets.zero,
                shrinkWrap: true,
                physics: NeverScrollableScrollPhysics(),
                itemBuilder: (context, index) {
                  GamesSum sum = _sportDetail.gamesSum[index];
                  return ConfigurableExpansionTile(
                    animatedWidgetFollowingHeader: arrowBottom(),
                    headerExpanded: _buildWrapListHeaderWidget(sum),
                    header: _buildWrapListHeaderWidget(sum),
                    children: [
                      _buildWrapListTileWidget("最近游玩：", "${sum.lastPlayAt}"),
                      _buildWrapListTileWidget("总局数：", "${sum.playCount} 局"),
                      _buildWrapListTileWidget("平均时长：", "${(sum.durationTotal / sum.playCount).floor()} min"),
                      _buildWrapListTileWidget("平均得分：", "${(sum.scoreTotal / sum.playCount).toStringAsFixed(1)}"),
                      _buildWrapListTileWidget("最高得分：", "${sum.scoreMax.toStringAsFixed(1)}"),
                      _buildWrapListTileWidget("运动效率：", "${(sum.consumeTotal / sum.durationTotal).toStringAsFixed(2)} kal/min"),
                      Padding(
                        padding: const EdgeInsets.all(8.0),
                        child: InkWell(
                          onTap: () => NavigatorUtil.goGameHistory(context, sum.game),
                          child: Row(
                            mainAxisSize: MainAxisSize.min,
                            children: <Widget>[
                              Text(
                                "查看记录详情  ",
                                style: Theme.of(context).textTheme.subtitle2.copyWith(color: Color(0xff666666)),
                              ),
                              arrowRight()
                            ],
                          ),
                        ),
                      )
                    ],
                  );
                },
                separatorBuilder: (context, index) => Divider(),
                itemCount: _sportDetail.gamesSum.length),
          if (_sportDetail != null && _sportDetail.achievements != null && _sportDetail.achievements.isNotEmpty)
            Column(
              children: <Widget>[
                Divider(),
                Space(
                  height: 10,
                ),
                Stack(
                  alignment: Alignment.center,
                  children: <Widget>[
                    Image.asset(
                      "lib/assets/img/mine_image_achievement.png",
                      fit: BoxFit.fitWidth,
                      width: 240,
                    ),
                    Text(
                      "我的成就",
                      style: Theme.of(context).textTheme.subtitle1,
                    )
                  ],
                ),
              ],
            ),
          if (_sportDetail != null && _sportDetail.achievements != null)
            Padding(
              padding: const EdgeInsets.all(12.0),
              child: achievementGroupWidget(_sportDetail?.achievements),
            ),
        ],
      ),
    );
  }

  Widget _buildWrapListHeaderWidget(GamesSum sum) {
    GameInfoData game = sum.game;
    return Row(
      children: <Widget>[
        Container(
          width: 50,
          height: 50,
          margin: EdgeInsets.fromLTRB(0, 6, 8, 6),
          decoration:
              BoxDecoration(borderRadius: BorderRadius.circular(6), image: DecorationImage(fit: BoxFit.cover, image: CachedNetworkImageProvider(game.cover))),
        ),
        Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: <Widget>[
            Text(
              game.name,
              style: Theme.of(context).textTheme.headline3,
            ),
            Space(
              height: 5,
            ),
            Text("时长: ${sum.durationTotal}分钟   消耗: ${sum.consumeTotal} kal")
          ],
        )
      ],
    );
  }

  Widget _buildWrapListTileWidget(String key, String value) {
    return Padding(
      padding: const EdgeInsets.symmetric(vertical: 4.0),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: <Widget>[Text(key), Text(value)],
      ),
    );
  }

  int toType() {
    return _TABS.indexOf(_tab.value);
  }

  void changeTab() {
    _valueNotifierSportDetail.value = null;

    DateTime time = _valueNotifierDate.value;
    int type = toType();
    Future<RespData<SportDetailSimple>> data;
    switch (type) {
      case 0:
        data = api.getSportRecordListOneDay('${time.year}-${'${time.month}'.padLeft(2, '0')}-${'${time.day}'.padLeft(2, '0')}');
        break;
      case 1:
        DateTime start = DateTime(time.year, time.month, time.day - time.weekday + 1);
        DateTime end = DateTime(time.year, time.month, time.day + 6 - time.weekday + 1);
        data = api.getSportRecordListByDay('${start.year}-${'${start.month}'.padLeft(2, '0')}-${'${start.day}'.padLeft(2, '0')}',
            '${end.year}-${'${end.month}'.padLeft(2, '0')}-${'${end.day}'.padLeft(2, '0')}');
        break;
      case 2:
        DateTime start = DateTime(time.year, time.month, 1);
        DateTime end = DateTime(time.year, time.month + 1, 0);
        data = api.getSportRecordListByDay('${start.year}-${'${start.month}'.padLeft(2, '0')}-${'${start.day}'.padLeft(2, '0')}',
            '${end.year}-${'${end.month}'.padLeft(2, '0')}-${'${end.day}'.padLeft(2, '0')}');
        break;
      case 3:
        data = api.getSportRecordListByMonth(time.year);
        break;
    }
    if (data != null) {
      data.then((value) => _valueNotifierSportDetail.value = SportDetail(
          recordsTodaySum: value.data.sum ?? RecordsTodaySum(consume: 0, duration: 0, crouchCount: 0, jumpCount: 0), recordsToday: value.data.records));
    }
  }

  void _showFirstTarget() async {
    SportTarget sportTarget;
    var result = await showDialog(
        context: context,
        barrierDismissible: false,
        builder: (context) => NotificationListener<TargetNotification>(
            onNotification: (notification) {
              sportTarget = notification.target;
              return true;
            },
            child: CustomAlertDialog(
              title: '运动目标设置',
              child: Column(
                children: <Widget>[
                  Align(
                    alignment: Alignment.centerLeft,
                    child: Padding(
                      padding: const EdgeInsets.only(left: 20.0),
                      child: Text(
                        "请选择每日运动目标",
                        style: Theme.of(context).textTheme.subtitle1,
                      ),
                    ),
                  ),
                  Padding(
                    padding: const EdgeInsets.symmetric(horizontal: 20.0, vertical: 5.0),
                    child: TargetModify(
                      editCustom: false,
                      padding: false,
                    ),
                  )
                ],
              ),
              ok: () async {
                if (sportTarget == null) {
                  return;
                }
                RespData<SportTargetToday> resp = (sportTarget.name == '自定义'
                    ? await api.setSportTargetCustom(sportTarget.duration, sportTarget.consume)
                    : await api.setSportTarget(sportTarget.id));
                if (resp.code == 0) {
                  Navigator.of(context).pop(resp.data);
                } else {
                  ToastUtil.show(resp.msg);
                }
              },
            )));
    if (result is SportTargetToday) {
      if (result != null) {
        setState(() {
          _valueNotifierTarget.value = result;
        });
      }
    }
  }

  void rollDate(int i) {
    if (_valueNotifierSportDetail.value == null) return;

    int type = toType();
    DateTime time = _valueNotifierDate.value;

    DateTime next;

    if (type == 0) {
      next = time.add(Duration(days: i));
    } else if (type == 1) {
      next = time.add(Duration(days: i > 0 ? 7 : -7));
    } else if (type == 2) {
      next = DateTime(time.year, time.month + i, 1);
    } else {
      next = time.add(Duration(days: i > 0 ? 366 : -366));
    }

    if (DateTime.now().isBefore(next)) {
      ToastUtil.show("还没有数据呢~");
      return;
    }

    _valueNotifierDate.value = next;
    changeTab();
  }
}

class _ProgressBar extends CustomPainter {
  final Paint _paint = Paint()
    ..color = Color(0xffeeeeee)
    ..isAntiAlias = true;
  final Paint _indicatorPaint = Paint()
    ..color = Color(0xffFFC400)
    ..isAntiAlias = true;

  double _paddingBar = 2;

  double percent;

  _ProgressBar(this.percent);

  @override
  void paint(Canvas canvas, Size size) {
    double indicator = size.width * min(1.0, this.percent);
    canvas.save();
    var rect = Rect.fromLTRB(0, size.height - 13, size.width, size.height);
    canvas.clipRRect(RRect.fromRectAndRadius(rect, Radius.circular(size.height / 2)), doAntiAlias: true);

    canvas.drawRect(rect, _paint);

    Paint _valuePaint = Paint()
      ..shader = LinearGradient(
        begin: Alignment.centerLeft,
        end: Alignment.centerRight,
        colors: <Color>[Color(0xffFFE600), Color(0xffFF9100)],
      ).createShader(rect);
    canvas.drawRect(Rect.fromLTRB(0, size.height - 13, size.width * this.percent, size.height), _valuePaint);

    canvas.restore();

    Path path = Path()
      ..moveTo(indicator, size.height - 13 - _paddingBar)
      ..lineTo(indicator - 5, 0)
      ..lineTo(indicator + 5, 0)
      ..close();
    canvas.drawPath(path, _indicatorPaint);
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    return false;
  }
}
