import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/material.dart';
import 'package:flutter_easyrefresh/easy_refresh.dart';
import 'package:flutter_staggered_grid_view/flutter_staggered_grid_view.dart';
import 'package:sport/bean/achievement_detail_info.dart';
import 'package:sport/provider/lib/provider_widget.dart';
import 'package:sport/provider/lib/simple_model.dart';
import 'package:sport/services/api/inject_api.dart';
import 'package:sport/widgets/appbar.dart';
import 'package:sport/widgets/loading.dart';
import 'package:sport/widgets/misc.dart';
import 'package:sport/widgets/space.dart';

class AchievementDetailPage extends StatefulWidget {
  final int id;

  AchievementDetailPage(this.id);


  @override
  State<StatefulWidget> createState() {
    return _PageState();
  }
}

class _PageState extends State<AchievementDetailPage> with InjectLoginApi {
  SimpleModel _model;

  @override
  void initState() {
    super.initState();
    _model = SimpleModel((page) async {
      return List()..add((await loginApi.getAchieveDetailInfo(widget.id)).data);
    });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        backgroundColor: Colors.white,
        body: ProviderWidget<SimpleModel>(
          model: _model,
          onModelReady: (model) => model.initData(),
          builder: (_, model, __) {
            AchieveDetailInfoData _data = model.list.isNotEmpty ? model.list.first : null;
            return EasyRefresh.custom(
              controller: model.refreshController,
              enableControlFinishRefresh: true,
              enableControlFinishLoad: true,
              // onRefresh: () => model.refresh(),
              header: buildClassicalHeader(),
              footer: buildClassicalFooter(),
              slivers: <Widget>[
                buildSliverAppBar(context, "成就详情"),
                if (model.isBusy)
                  SliverToBoxAdapter(
                    child: RequestLoadingWidget(),
                  ),
                if (model.isIdle)
                  SliverToBoxAdapter(
                      child: Padding(
                    padding: const EdgeInsets.all(12.0),
                    child: Row(
                      children: <Widget>[
                        CircleAvatar(
                          backgroundImage: CachedNetworkImageProvider(_data.logo),
                          radius: 45,
                        ),
                        Space(
                          width: 12,
                        ),
                        Column(
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: <Widget>[
                            Padding(
                              padding: const EdgeInsets.symmetric(vertical: 2.0),
                              child: Row(
                                children: <Widget>[
                                  Text("成就名称：", style: Theme.of(context).textTheme.subtitle1),
                                  Text("${_data.name}", style: Theme.of(context).textTheme.bodyText2)
                                ],
                              ),
                            ),
                            Padding(
                              padding: const EdgeInsets.symmetric(vertical: 2.0),
                              child: Row(
                                children: <Widget>[
                                  Text("成就条件：", style: Theme.of(context).textTheme.subtitle1),
                                  Text("累计运动${_data.conditionDuration}分钟", style: Theme.of(context).textTheme.bodyText2)
                                ],
                              ),
                            ),
                            Padding(
                                padding: const EdgeInsets.symmetric(vertical: 2.0),
                                child: Row(
                                  children: <Widget>[
                                    Text("成就奖励：", style: Theme.of(context).textTheme.subtitle1),
                                    Text("积分${_data.rewardScore}", style: Theme.of(context).textTheme.bodyText2)
                                  ],
                                )),
                            Padding(
                              padding: const EdgeInsets.symmetric(vertical: 2.0),
                              child: Row(
                                children: <Widget>[
                                  Text("获取时间：", style: Theme.of(context).textTheme.subtitle1),
                                  Text("${_data.createdAt == null ? "未获得" : _data.createdAt}", style: Theme.of(context).textTheme.bodyText2)
                                ],
                              ),
                            ),
                          ],
                        ),
                      ],
                    ),
                  )),
                if (model.isIdle)
                  SliverToBoxAdapter(
                      child: Padding(
                    padding: const EdgeInsets.all(12.0),
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: <Widget>[
                        Divider(),
                        Text("关联成就", style: Theme.of(context).textTheme.headline3),
                      ],
                    ),
                  )),
                if (model.isIdle && _data.relateAchivements.isNotEmpty)
                  SliverToBoxAdapter(
                    child: StaggeredGridView.countBuilder(
                      padding: EdgeInsets.zero,
                      shrinkWrap: true,
                      physics: NeverScrollableScrollPhysics(),
                      crossAxisCount: 4,
                      itemCount: _data.relateAchivements.length,
                      itemBuilder: (BuildContext context, int index) => achievementWidget(context, _data.relateAchivements[index], replace: true),
                      mainAxisSpacing: 12.0,
                      crossAxisSpacing: 12.0,
                      staggeredTileBuilder: (int index) => StaggeredTile.fit(1),
                    ),
                  )
              ],
            );
          },
        ));
  }
}
