import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:sport/bean/achievement_info.dart';
import 'package:sport/provider/achievement_info_model.dart';
import 'package:sport/provider/user_model.dart';
import 'package:sport/router/navigator_util.dart';
import 'package:sport/router/routes.dart';
import 'package:sport/widgets/appbar.dart';
import 'package:sport/widgets/border_radius.dart';
import 'package:sport/widgets/box.dart';
import 'package:sport/widgets/image.dart';
import 'package:sport/widgets/loading.dart';
import 'package:sport/widgets/space.dart';

class AchievementPage extends StatefulWidget {
  @override
  State<StatefulWidget> createState() => _PageState();
}

class _PageState extends State<AchievementPage> {
  AchievementInfoModel _achievementInfoModel = new AchievementInfoModel(1);
  bool _isLoading = true;
  List<String> _infoImgs = [
    "https://ss0.bdstatic.com/70cFvHSh_Q1YnxGkpoWK1HF6hhy/it/u=349436276,2897896306&fm=15&gp=0.jpg",
    "https://ss3.bdstatic.com/70cFv8Sh_Q1YnxGkpoWK1HF6hhy/it/u=3020158327,1401208296&fm=26&gp=0.jpg",
    "https://ss1.bdstatic.com/70cFvXSh_Q1YnxGkpoWK1HF6hhy/it/u=1405015388,2107694993&fm=26&gp=0.jpg",
    "https://ss0.bdstatic.com/70cFvHSh_Q1YnxGkpoWK1HF6hhy/it/u=242101413,2188657869&fm=26&gp=0.jpg",
    "https://dss3.bdstatic.com/70cFv8Sh_Q1YnxGkpoWK1HF6hhy/it/u=3971074399,97839074&fm=26&gp=0.jpg",
  ];
  @override
  void initState() {
    // TODO: implement initState
    super.initState();
    initData();
  }

  initData() async {
    await _achievementInfoModel.getAchievementInfo();
    setState(() {
      _isLoading = false;
    });
  }

  List<Widget> getAchievement(int len, AchievementInfoData _achievementInfo) {
    List<Widget> achievements = [];

    for (var i = 0; i < len; i++) {
      achievements.add(InkWell(
        onTap: () {
          NavigatorUtil.go(context,
              "${Routes.achievementDetail}?id=${_achievementInfo.achievements[i].id}");
        },
        child: Column(
          children: <Widget>[
            WrapBorderRadius(
              50.0,
              width: 66.0,
              height: 66.0,
              color: Colors.grey,
              img: DecorationImage(image: CachedNetworkImageProvider(_infoImgs[i % 5])),
            ),
            Padding(
              padding: EdgeInsets.only(top: 12.0),
              child: Text(_achievementInfo.achievements[i].name),
            )
          ],
        ),
      ));
    }
    return achievements;
  }

  @override
  Widget build(BuildContext context) {
    return !_isLoading
        ? Scaffold(
            // provider 只能有一个？
            body: ChangeNotifierProvider<AchievementInfoModel>(
                create: (context) => _achievementInfoModel,
                child: Consumer<AchievementInfoModel>(
                    builder: (context, _achievementInfo, child) {
                  AchievementInfoData _data = _achievementInfo.data.data;
                  return CustomScrollView(slivers: <Widget>[
                    buildSliverAppBar(context, "我的等级",
                        backgroundColor:
                            Theme.of(context).scaffoldBackgroundColor),
                    SliverToBoxAdapter(
                        child: BoxWidget(
                            body: Column(
                      children: <Widget>[
                        Padding(
                            padding:
                                const EdgeInsets.only(top: 16.0, bottom: 8.0),
                            child: Consumer<UserModel>(
                              builder: (_, model, __) {
                                return Container(
                                  width: 40.0,
                                  height: 40.0,
                                  child: CircleAvatar(
                                      radius: 100,
                                      backgroundImage: model.user.avatar != null
                                          ? CachedNetworkImageProvider(model.user.avatar)
                                          : ""),
                                );
                              },
                            )),
                        Text(
                          "${_data.user.level}级",
                          style: Theme.of(context).textTheme.headline3,
                        ),
                        Space(
                          height: 24,
                        ),
                        ClipRRect(
                          borderRadius: BorderRadius.circular(10),
                          child: Container(
                            child: LinearProgressIndicator(
                              value: _data.user.score /
                                  (_data.extra.nextLevel.diff +
                                      _data.user.score),
                            ),
                            height: 7,
                          ),
                        ),
                        Space(
                          height: 8,
                        ),
                        Text(
                          "总积分：${_data.user.score}",
                          style: Theme.of(context)
                              .textTheme
                              .subtitle1
                              .copyWith(color: Theme.of(context).accentColor),
                        ),
                        Space(
                          height: 4,
                        ),
                        Text(
                          "再获得 ${_data.extra.nextLevel.diff} 积分可以晋级下一个段位",
                          style: Theme.of(context).textTheme.bodyText1,
                        ),
                        Divider(),
                        Padding(
                          padding: const EdgeInsets.symmetric(vertical: 8.0),
                          child: Row(
                            children: <Widget>[
                              Expanded(
                                child: Text(
                                  "我的成就",
                                  style: Theme.of(context).textTheme.subtitle1,
                                ),
                              ),
                              Text(
                                "查看完整榜单 ",
                                style: Theme.of(context).textTheme.bodyText1,
                              ),
                              arrowRight4()
                            ],
                          ),
                        ),
                        Padding(
                          padding: EdgeInsets.symmetric(vertical: 20.0),
                          child: Wrap(
                            spacing: 10.0,
                            runSpacing: 15,
                            children: getAchievement(
                                _data.achievements.length, _data),
                          ),
                        )
                      ],
                    ))),
                    SliverToBoxAdapter(
                        child: Center(
                      child: Padding(
                        padding: const EdgeInsets.all(8.0),
                        child: Text("未获得成就",
                            style: Theme.of(context).textTheme.bodyText2),
                      ),
                    )),
                    SliverList(
                        delegate: SliverChildBuilderDelegate((content, index) {
                      return InkWell(
                        onTap: () =>
                            NavigatorUtil.go(context, "${Routes.achievementDetail}?id=${_data.remainAchievements[index].id}"),
                        child: BoxWidget(
                          body: Row(
                            children: <Widget>[
                              Column(
                                children: <Widget>[
                                  CircleAvatar(
                                    backgroundImage:
                                    CachedNetworkImageProvider(_infoImgs[index % 5]),
                                    radius: 30,
                                  ),
                                  Space(
                                    height: 4,
                                  ),
                                  Text("未获得",
                                      style:
                                          Theme.of(context).textTheme.bodyText1)
                                ],
                              ),
                              Space(
                                width: 12,
                              ),
                              Column(
                                crossAxisAlignment: CrossAxisAlignment.start,
                                children: <Widget>[
                                  Row(
                                    children: <Widget>[
                                      Text("成就名称：",
                                          style: Theme.of(context)
                                              .textTheme
                                              .subtitle1),
                                      Text(
                                          "${_data.remainAchievements[index].name}",
                                          style: Theme.of(context)
                                              .textTheme
                                              .bodyText2
                                              .copyWith(
                                                  color: Theme.of(context)
                                                      .accentColor))
                                    ],
                                  ),
                                  Space(
                                    height: 6,
                                  ),
                                  Row(
                                    children: <Widget>[
                                      Text("成就条件：",
                                          style: Theme.of(context)
                                              .textTheme
                                              .subtitle1),
                                      Text(
                                          "累计运动${_data.remainAchievements[index].conditionSportDuration}分钟",
                                          style: Theme.of(context)
                                              .textTheme
                                              .bodyText2)
                                    ],
                                  ),
                                  Space(
                                    height: 6,
                                  ),
                                  Row(
                                    children: <Widget>[
                                      Text("成就奖励：",
                                          style: Theme.of(context)
                                              .textTheme
                                              .subtitle1),
                                      Text(
                                          "积分+${_data.remainAchievements[index].rewardScore}",
                                          style: Theme.of(context)
                                              .textTheme
                                              .bodyText2)
                                    ],
                                  )
                                ],
                              ),
                            ],
                          ),
                        ),
                      );
                    }, childCount: _data.remainAchievements.length))
                  ]);
                })))
        : FadeingCircleLoading(context, backgroundColor: Colors.white);
  }
}
