import 'dart:math';

import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_easyrefresh/easy_refresh.dart';
import 'package:flutter_staggered_grid_view/flutter_staggered_grid_view.dart';
import 'package:provider/provider.dart';
import 'package:sport/bean/achievement_info.dart';
import 'package:sport/bean/rank_game_info.dart';
import 'package:sport/bean/user.dart';
import 'package:sport/provider/lib/provider_widget.dart';
import 'package:sport/provider/lib/simple_model.dart';
import 'package:sport/provider/user_model.dart';
import 'package:sport/router/navigator_util.dart';
import 'package:sport/router/routes.dart';
import 'package:sport/services/api/inject_api.dart';
import 'package:sport/widgets/appbar.dart';
import 'package:sport/widgets/box.dart';
import 'package:sport/widgets/image.dart';
import 'package:sport/widgets/loading.dart';
import 'package:sport/widgets/misc.dart';
import 'package:sport/widgets/space.dart';

class LevelPage extends StatefulWidget {
  @override
  State<StatefulWidget> createState() => _PageState();
}

class _PageState extends State<LevelPage> with InjectLoginApi {
  SimpleModel _model;
  bool _max = false;

  @override
  void initState() {
    // TODO: implement initState
    super.initState();
    _model = SimpleModel((page) async => [(await loginApi.getAchievementInfo()).data]);
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        body: ProviderWidget<SimpleModel>(
      model: _model,
      onModelReady: (model) => model.initData(),
      builder: (_, model, __) {
        AchievementInfoData _data = model.list.isNotEmpty ? model.list.first : null;
        return EasyRefresh.custom(
          controller: model.refreshController,
          enableControlFinishRefresh: true,
          enableControlFinishLoad: true,
          onRefresh: () => model.refresh(),
          header: buildClassicalHeader(),
          slivers: <Widget>[
            buildSliverAppBar(context, "我的等级", backgroundColor: Theme.of(context).scaffoldBackgroundColor),
            if (model.isBusy)
              SliverToBoxAdapter(
                child: RequestLoadingWidget(),
              ),
            if (model.isIdle && _data != null)
              SliverToBoxAdapter(
                  child: BoxWidget(
                      body: Column(
                children: <Widget>[
                  Padding(
                      padding: const EdgeInsets.only(top: 16.0, bottom: 8.0),
                      child: Container(
                        width: 40.0,
                        height: 40.0,
                        child: CircleAvatar(
                            radius: 100,
                            backgroundImage: CachedNetworkImageProvider(_data.user != null && _data.user.levelInfo != null ? _data.user.levelInfo.logo : "")),
                      )),
                  Text(
                    "${_data.user.level}级",
                    style: Theme.of(context).textTheme.headline3,
                  ),
                  Space(
                    height: 24,
                  ),
                  ClipRRect(
                    borderRadius: BorderRadius.circular(10),
                    child: Container(
                      child: LinearProgressIndicator(
                        value: _data.user.score / (_data.extra.nextLevel.diff + _data.user.score),
                      ),
                      height: 7,
                    ),
                  ),
                  Space(
                    height: 8,
                  ),
                  Text(
                    "总积分：${_data.user.score}",
                    style: Theme.of(context).textTheme.subtitle1.copyWith(color: Theme.of(context).accentColor),
                  ),
                  Space(
                    height: 4,
                  ),
                  Text(
                    "再获得 ${_data.extra.nextLevel.diff} 积分可以晋级下一个段位",
                    style: Theme.of(context).textTheme.bodyText1,
                  ),
                  Space(
                    height: 8,
                  ),
                  Divider(),
                  Padding(
                    padding: const EdgeInsets.symmetric(vertical: 8.0),
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      children: <Widget>[
                        Text(
                          "我的成就",
                          style: Theme.of(context).textTheme.subtitle1,
                        ),
                        if (_data.user.achievements.length > 4 || _max)
                          InkWell(
                            onTap: () {
                              setState(() {
                                _max = true;
                              });
                            },
                            child: Row(
                              children: <Widget>[
                                Text(
                                  "查看全部  ",
                                  style: Theme.of(context).textTheme.bodyText1,
                                ),
                                arrowRight4()
                              ],
                            ),
                          ),
                      ],
                    ),
                  ),
                  if (_data.user.achievements.isEmpty) Padding(padding: const EdgeInsets.all(24.0), child: Text("您还未获得成就", style: Theme.of(context).textTheme.bodyText1)),
                  StaggeredGridView.countBuilder(
                    padding: EdgeInsets.zero,
                    shrinkWrap: true,
                    physics: NeverScrollableScrollPhysics(),
                    crossAxisCount: 4,
                    itemCount: _max ? _data.user.achievements.length : min(4, _data.user.achievements.length),
                    itemBuilder: (BuildContext context, int index) => achievementWidget(context, _data.user.achievements[index]),
                    mainAxisSpacing: 12.0,
                    crossAxisSpacing: 12.0,
                    staggeredTileBuilder: (int index) => StaggeredTile.fit(1),
                  )
                ],
              ))),
            if (model.isIdle)
              SliverToBoxAdapter(
                  child: Center(
                child: Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: Text("未获得成就", style: Theme.of(context).textTheme.bodyText2),
                ),
              )),
            if (model.isIdle && _data.remainAchievements.isNotEmpty)
              SliverList(
                  delegate: SliverChildBuilderDelegate((content, index) {
                Achievement item = _data.remainAchievements[index];
                return BoxWidget(
                  body: InkWell(
                    onTap: () => NavigatorUtil.goAchievementDetails(context, id: item.id),
                    child: Row(
                      children: <Widget>[
                        Column(
                          children: <Widget>[
                            ColorFiltered(
                              colorFilter: ColorFilter.mode(Colors.grey, BlendMode.modulate),
                              child: CircleAvatar(
                                backgroundImage: CachedNetworkImageProvider(item.logo),
                                radius: 30,
                              ),
                            ),
                            Space(
                              height: 5,
                            ),
                            Text("未获得", style: Theme.of(context).textTheme.bodyText1)
                          ],
                        ),
                        Space(
                          width: 12,
                        ),
                        Column(
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: <Widget>[
                            Row(
                              children: <Widget>[
                                Text("成就名称：", style: Theme.of(context).textTheme.subtitle1),
                                Text("${item.name}", style: Theme.of(context).textTheme.bodyText2.copyWith(color: Theme.of(context).accentColor))
                              ],
                            ),
                            Space(
                              height: 6,
                            ),
                            Row(
                              children: <Widget>[
                                Text("成就条件：", style: Theme.of(context).textTheme.subtitle1),
                                Text("累计运动${item.conditionSportDuration}分钟", style: Theme.of(context).textTheme.bodyText2)
                              ],
                            ),
                            Space(
                              height: 6,
                            ),
                            Row(
                              children: <Widget>[
                                Text("成就奖励：", style: Theme.of(context).textTheme.subtitle1),
                                Text("积分+${item.rewardScore}", style: Theme.of(context).textTheme.bodyText2)
                              ],
                            )
                          ],
                        ),
                      ],
                    ),
                  ),
                );
              }, childCount: _data.remainAchievements.length))
          ],
        );
      },
    ));
  }
}
