import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:provider/provider.dart';
import 'package:sport/provider/user_model.dart';
import 'package:sport/router/navigator_util.dart';
import 'package:sport/router/routes.dart';
import 'package:sport/services/api/inject_api.dart';
import 'package:sport/widgets/image.dart';
import 'package:sport/widgets/list_tile.dart';
import 'package:sport/widgets/space.dart';

class MyPage extends StatefulWidget {
  @override
  _MyPageState createState() => _MyPageState();
}

class _MyPageState extends State<MyPage> with InjectApi {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return AnnotatedRegion<SystemUiOverlayStyle>(
      value: SystemUiOverlayStyle.light,
      child: Material(
        child: Scaffold(
          backgroundColor: Colors.white,
          body: Stack(
            children: <Widget>[
              // 放外面一点的吧
              InkWell(
                onTap: () => NavigatorUtil.go(context, Routes.userInfo),
                child: AspectRatio(
                    aspectRatio: 1125 / 435.0,
                    child: Stack(
                      fit: StackFit.loose,
                      children: <Widget>[
                        Image.asset("lib/assets/img/bg_userprofile.jpg"),
                        SafeArea(
                          child: Container(
                            padding: EdgeInsets.all(24.0),
                            child: Row(
                              children: <Widget>[
                                Consumer<UserModel>(
                                  builder: (_, model, __) {
                                    return CircleAvatar(
                                      backgroundImage: userAvatarProvider(model.user.avatar),
                                      radius: 30,
                                    );
                                  },
                                ),
                                Space(
                                  width: 12,
                                ),
                                Consumer<UserModel>(
                                  builder: (_, model, __) => Column(
                                    mainAxisAlignment: MainAxisAlignment.center,
                                    crossAxisAlignment: CrossAxisAlignment.start,
                                    children: <Widget>[
                                      model.user == null ? Container() : Text(model.user.name ?? "", style: TextStyle(color: Colors.white, fontSize: 18)),
                                      Space(
                                        height: 7,
                                      ),
                                      Row(
                                        children: <Widget>[
                                          Row(
                                            children: <Widget>[
                                              Image.asset("lib/assets/img/mine_icon_${model.user.gender == 1 ? "manl" : "girl"}_white.png"),
                                              Space(
                                                width: 4,
                                              ),
                                              Text(
                                                model.user.gender == 1 ? "男" : "女",
                                                style: TextStyle(color: Colors.white, fontSize: 14),
                                              ),
                                            ],
                                          ),
                                          if ((model.user.age ?? 0) != 0)
                                            Padding(
                                              padding: const EdgeInsets.only(left: 10.0),
                                              child: Text(
                                                "${model.user.age}岁",
                                                style: TextStyle(color: Colors.white, fontSize: 14),
                                              ),
                                            ),
                                          if (model.user.city != null)
                                            Padding(
                                              padding: const EdgeInsets.only(left: 10.0),
                                              child: Text(
                                                "${model.user.province}${model.user.province == model.user.city ? '' : model.user.city}",
                                                style: TextStyle(color: Colors.white, fontSize: 14),
                                              ),
                                            ),
                                        ],
                                      )
                                    ],
                                  ),
                                )
                              ],
                            ),
                          ),
                        ),
                      ],
                    )),
              ),
              Column(
                children: <Widget>[
                  SizedBox(
                    height: MediaQuery.of(context).size.width * 435.0 / 1125 - 10,
                  ),
                  Container(
                    height: 11,
                    decoration: BoxDecoration(borderRadius: BorderRadius.vertical(top: Radius.circular(10)), color: Colors.white),
                  ),
                  ListView(
                    padding: EdgeInsets.symmetric(horizontal: 12.0, vertical: 0),
                    physics: NeverScrollableScrollPhysics(),
                    shrinkWrap: true,
                    children: divideTiles(
                      context: context,
                      includeLast: true,
                      tiles: [
                        ListTile(
                          leading: Padding(padding: EdgeInsets.only(bottom: 10, top: 5), child: Image.asset("lib/assets/img/mine_image_shoe.png")),
                          title: Padding(
                              padding: EdgeInsets.fromLTRB(12.0, 0, 12.0, 10),
                              child: Column(
                                crossAxisAlignment: CrossAxisAlignment.start,
                                children: <Widget>[
                                  Text(
                                    "剩余电量70%",
                                    style: Theme.of(context).textTheme.subtitle1.copyWith(fontSize: 12),
                                  ),
                                  Space(
                                    height: 8,
                                  ),
                                  ClipRRect(
                                    borderRadius: BorderRadius.circular(10),
                                    child: Container(
                                      child: LinearProgressIndicator(
                                        value: 0.7,
                                        valueColor: AlwaysStoppedAnimation(Color.fromRGBO(0, 220, 66, 1)
//                                            Color(0xffFF5B1D)
                                            ),
                                      ),
                                      height: 7,
                                    ),
                                  )
                                ],
                              )),
                          trailing: arrowRight5(),
                          contentPadding: EdgeInsets.symmetric(horizontal: 0.0),
                          onTap: () {
                            NavigatorUtil.go(context, Routes.deviceInfo);
                          },
                        ),
                        ListTile(
                          title: Text("我的等级"),
                          contentPadding: EdgeInsets.symmetric(horizontal: 0.0),
                          trailing: arrowRight5(),
                          onTap: () => NavigatorUtil.go(context, Routes.level),
                        ),
                        ListTile(
                          title: Text("我的运动"),
                          contentPadding: EdgeInsets.symmetric(horizontal: 0.0),
                          trailing: arrowRight5(),
                          onTap: () => NavigatorUtil.go(context, Routes.gamelistpage),
                        ),
                        ListTile(
                          title: Text("用户反馈"),
                          contentPadding: EdgeInsets.symmetric(horizontal: 0.0),
                          trailing: arrowRight5(),
                          onTap: () => NavigatorUtil.go(context, Routes.feedback),
                        ),
                        ListTile(
                          title: Text("设置"),
                          contentPadding: EdgeInsets.symmetric(horizontal: 0.0),
                          trailing: arrowRight5(),
                          onTap: () => NavigatorUtil.go(context, Routes.setting),
                        ),
                      ],
                    ).toList(),
                  )
                ],
              ),
            ],
          ),
        ),
      ),
    );
  }
}
