import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:provider/provider.dart';
import 'package:sport/bean/comment.dart';
import 'package:sport/pages/social/notification.dart';
import 'package:sport/provider/user_model.dart';
import 'package:sport/services/api/inject_api.dart';
import 'package:sport/utils/toast.dart';
import 'package:sport/widgets/dialog/alert_dialog.dart';
import 'package:sport/widgets/dialog/comment_dialog.dart';
import 'package:sport/widgets/dialog/modal_bottom_action.dart';
import 'package:sport/widgets/dialog/request_dialog.dart';
import 'package:sport/widgets/image.dart';
import 'package:sport/widgets/misc.dart';
import 'package:sport/widgets/space.dart';

class PostCommentWidget extends StatefulWidget {
  final Comment comment;

  PostCommentWidget(this.comment);

  @override
  State<StatefulWidget> createState() => _PostCommentState();
}

class _PostCommentState extends State<PostCommentWidget> with InjectApi {
  var _padding = Space(
    width: 5,
  );

  String _selfId;

  @override
  void initState() {
    super.initState();
    _selfId = "${Provider.of<UserModel>(context, listen: false).user.id}";
  }

  @override
  Widget build(BuildContext context) {
    Comment item = widget.comment;
    return Column(
      children: <Widget>[
        Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: <Widget>[
            Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: <Widget>[
                buildSocialUserWidget(context, item?.socialInfo, item.createTime),
                PopupMenuButton(
                  icon: arrowBottom(),
                  onSelected: (action) async {
                    switch (action) {
                      case "report":
                        request(context, () async {
                          await api.postForumReport(commentId: item.id).catchError((onError) {});
                          ToastUtil.show("举报成功");
                        });
                        break;
                      case "block":
                        request(context, () async {
                          await api.postForumBlockObject(item.id, "comment").catchError((onError) {});
                          ToastUtil.show("屏蔽成功");
                          CommentNotification(item, CommentNotification.TYPE_DEL).dispatch(context);
                        });
                        break;
                      case "delete":
                        if (await showDialog(
                              context: context,
                              builder: (context) => CustomAlertDialog(title: '确定删除此条？', ok: () => Navigator.of(context).pop(true)),
                            ) ==
                            true) {
                          request(context, () async {
                            await api.postDelComment(item.id);
                            ToastUtil.show("删除成功");
                            CommentNotification(item, CommentNotification.TYPE_DEL).dispatch(context);
                          });
                        }
                        break;
                    }
                  },
                  itemBuilder: (_) => [
                    if (item.userId != _selfId)
                      PopupMenuItem(
                        value: "report",
                        child: Text("举报"),
                      ),
                    if (item.userId != _selfId)
                      PopupMenuItem(
                        value: "block",
                        child: Text("屏蔽此条"),
                      ),
                    if (item.userId == _selfId)
                      PopupMenuItem(
                        value: "delete",
                        child: Text("删除此条"),
                      )
                  ],
                )
              ],
            ),
            Padding(
              padding: const EdgeInsets.symmetric(horizontal: 12.0, vertical: 6.0),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: <Widget>[
                  GestureDetector(
                    onTap: () {
                      CommentInputNotification(item).dispatch(context);
                    },
                    onLongPress: () async {
                      await showActionDialog(context, {
                        "复制文字": () {
                          Clipboard.setData(ClipboardData(text: item.content));
                          ToastUtil.show("已复制到粘贴板");
                        }
                      });
                    },
                    child: Padding(
                        padding: const EdgeInsets.only(bottom: 12.0),
                        child: Column(crossAxisAlignment: CrossAxisAlignment.start, children: <Widget>[
                          item.isExpansion ?? false == false
                              ? RichText(
                                  maxLines: 5,
                                  text: TextSpan(style: DefaultTextStyle.of(context).style, children: <InlineSpan>[
                                    if (item.toUserName != null && item.toUserName.isNotEmpty)
                                      TextSpan(
                                        text: '@${item.toUserName} ',
                                        style: Theme.of(context).textTheme.subtitle1.copyWith(fontSize: 16, color: Theme.of(context).accentColor),
                                      ),
                                    TextSpan(text: item.content, style: Theme.of(context).textTheme.subtitle1.copyWith(fontSize: 16)),
                                  ]),
                                )
                              : RichText(
                                  text: TextSpan(style: DefaultTextStyle.of(context).style, children: <InlineSpan>[
                                    if (item.toUserName != null && item.toUserName.isNotEmpty)
                                      TextSpan(
                                        text: '@${item.toUserName} ',
                                        style: Theme.of(context).textTheme.subtitle1.copyWith(fontSize: 16, color: Theme.of(context).accentColor),
                                      ),
                                    TextSpan(text: item.content, style: Theme.of(context).textTheme.subtitle1.copyWith(fontSize: 16)),
                                  ]),
                                ),
                          if (isExpansion(item.content))
                            GestureDetector(
                                onTap: () {
                                  item.isExpansion = !(item.isExpansion ?? true);
                                  setState(() {});
                                },
                                child: Text(item.isExpansion ?? false == false ? "全部" : "收起", style: Theme.of(context).textTheme.bodyText2)),
                        ])),
                  ),
                  Row(
                    children: <Widget>[
                      Expanded(
                          child: item.commentCount == 0
                              ? Container()
                              : InkWell(
                                  onTap: () {
                                    _showCommentList(context, item);
                                  },
                                  child: Row(
                                    crossAxisAlignment: CrossAxisAlignment.center,
                                    children: <Widget>[Text("查看${item.commentCount}条回复"), _padding, arrowRight4()],
                                  ))),
                      GestureDetector(
                        behavior: HitTestBehavior.opaque,
                        onTap: () async {
                          await (item.isLiked ?? false ? api.postForumUnLike(item.id, "comment_id") : api.postForumLike(item.id, "comment_id"));
                          setState(() {
                            item.toggleLike();
                          });
                        },
                        child: SizedBox(
                          height: 19,
                          child: Row(
                            children: <Widget>[
                              Image.asset("lib/assets/img/bbslist_icon_like${item.isLiked ? 'd' : ''}.png"),
                              _padding,
                              Text(
                                item.likeCount == 0 ? "点赞" : "${item.likeCount}",
                                strutStyle: StrutStyle(forceStrutHeight: true),
                                style: Theme.of(context).textTheme.bodyText1,
                              ),
                            ],
                          ),
                        ),
                      ),
                      Space(
                        width: 16,
                      ),
                      InkWell(
                        onTap: () => _showCommentList(context, item, comment: true),
                        child: Row(
                          children: <Widget>[
                            Image.asset("lib/assets/img/bbslist_icon_reply.png"),
                            _padding,
                            Text(
                              "回复",
                              strutStyle: StrutStyle(forceStrutHeight: true),
                              style: Theme.of(context).textTheme.bodyText1,
                            ),
                          ],
                        ),
                      ),
                    ],
                  ),
                ],
              ),
            ),
          ],
        ),
        Divider()
      ],
    );
  }

  bool isExpansion(String text) {
    TextPainter _textPainter = TextPainter(maxLines: 5, text: TextSpan(text: text, style: TextStyle(fontSize: 16.0)), textDirection: TextDirection.ltr)
      ..layout(maxWidth: MediaQuery.of(context).size.width - 24);
    if (_textPainter.didExceedMaxLines) {
      return true;
    } else {
      return false;
    }
  }

  void _showCommentList(BuildContext context, Comment currentItem, {bool comment = false}) async {
    await showCommentList(context, currentItem, comment: comment);
    setState(() {});
  }
}
