import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:multi_image_picker/multi_image_picker.dart';
import 'package:path_provider/path_provider.dart';
import 'package:sport/bean/image.dart' as photo;
import 'package:sport/services/api/inject_api.dart';
import 'package:sport/utils/toast.dart';
import 'package:sport/widgets/appbar.dart';
import 'package:sport/widgets/button_primary.dart';
import 'package:sport/widgets/dialog/alert_dialog.dart';
import 'package:sport/widgets/space.dart';

class PostPage extends StatefulWidget {
  final String id;

  const PostPage(this.id);

  @override
  State<StatefulWidget> createState() => _PageState();
}

class _PageState extends State<PostPage> {
  List<Asset> imageList = [];
  TextEditingController _controller;
  ValueNotifier<String> _valueNotifier = ValueNotifier("");
  FocusNode _focusNode;

  @override
  void initState() {
    super.initState();
    _focusNode = FocusNode();
    _controller = TextEditingController()..addListener(() {});
  }

  @override
  void dispose() {
    super.dispose();
    _controller?.dispose();
    _focusNode?.dispose();
    _valueNotifier?.dispose();
    PaintingBinding.instance.imageCache.clear();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.white,
      appBar: AppBar(
        leading: buildBackButton(context),
        title: Row(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: <Widget>[
            Text(""),
            PrimaryButton(
              width: 65,
              height: 35,
              content: "发布",
              callback: () async {
                String postValue = _valueNotifier.value.trim();
                if (postValue == "") {
                  ToastUtil.show("不能发布空白内容喔！");
                  return;
                }

                if (await showDialog(
                      context: context,
                      builder: (context) => CustomAlertDialog(title: '是否确认发布', ok: () => Navigator.of(context).pop(true)),
                    ) !=
                    true) {
                  return;
                }

                bool result = await showDialog(
                    context: context,
                    barrierDismissible: false,
                    builder: (context) => SimpleDialog(
                          children: <Widget>[PostAction(widget.id, postValue, imageList)],
                        ));
                if (result == true) {
                  ToastUtil.show("发布成功");
                  Navigator.of(context).pop(true);
                } else {
                  // ToastUtil.show("已取消发布");
                }
              },
            )
          ],
        ),
      ),
      body: SingleChildScrollView(
        child: Padding(
          padding: const EdgeInsets.symmetric(horizontal: 12.0, vertical: 6.0),
          child: Form(
            onWillPop: () async {
              if (_valueNotifier.value.isNotEmpty || imageList.isNotEmpty) {
                bool result = await showDialog(
                        context: context,
                        barrierDismissible: false,
                        builder: (context) {
                          return CustomAlertDialog(
                            title: '确认关闭吗？',
                            ok: () {
                              Navigator.of(context).pop(true);
                            },
                          );
                        }) ??
                    false;
                return result;
              }
              return true;
            },
            child: Column(
              children: <Widget>[
                TextFormField(
                  focusNode: _focusNode,
                  controller: _controller,
                  keyboardType: TextInputType.multiline,
                  maxLines: 8,
                  maxLength: 500,
                  style: TextStyle(fontSize: 16),
                  strutStyle: StrutStyle(forceStrutHeight: true, height: 1.4),
                  onChanged: (v) {
                    _valueNotifier.value = v;
                    if (_valueNotifier.value.length == 500) {
                      ToastUtil.show("文字数量已达上限");
                    }
                  },
                  buildCounter: (
                    BuildContext context, {
                    int currentLength,
                    int maxLength,
                    bool isFocused,
                  }) {
                    return Align(alignment: Alignment.centerLeft, child: Text("$currentLength/$maxLength"));
                  },
                  decoration: InputDecoration(hintText: '发表你的看法...', border: InputBorder.none),
                ),
                Space(
                  height: 16,
                ),
                GridView.builder(
                    padding: EdgeInsets.zero,
                    shrinkWrap: true,
                    physics: NeverScrollableScrollPhysics(),
                    itemCount: imageList.length + (imageList.length < 9 ? 1 : 0),
                    gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(crossAxisCount: 3, crossAxisSpacing: 12.0, mainAxisSpacing: 12.0),
                    itemBuilder: (context, index) {
                      return InkWell(
                        onTap: () => _select(),
                        child: ClipRRect(
                          borderRadius: BorderRadius.circular(6),
                          child: index >= imageList.length
                              ? Image.asset(
                                  "lib/assets/img/bbs_icon_addimage.png",
                                  fit: BoxFit.cover,
                                )
                              : AssetThumb(
                                  asset: imageList[index],
                                  quality: 50,
                                  width: 200,
                                  height: 200,
                                ),
                        ),
                      );
                    }),
                Space(
                  height: 300,
                ),
              ],
            ),
          ),
        ),
      ),
    );
  }

  void _select() async {
    _focusNode?.unfocus();
    List<Asset> resultList;
    String error;

//    int max = 9 - imageList.length;
//    if (max <= 0) {
//      Fluttertoast.showToast(msg: "不能再添加了~", backgroundColor: Colors.black, textColor: Colors.white, fontSize: 16.0);
//      return;
//    }

    try {
      resultList = await MultiImagePicker.pickImages(
          maxImages: 9,
          selectedAssets: imageList,
          materialOptions: MaterialOptions(
            actionBarTitle: "选择图片",
            allViewTitle: "选择图片",
            useDetailsView: true,
            startInAllView: true,
            selectionLimitReachedText: "不能选择更多了~",
          ),
          cupertinoOptions: CupertinoOptions(
            selectionFillColor: "#ff11ab",
            selectionTextColor: "#ffffff",
            selectionCharacter: "✓",
          ));
    } on Exception catch (e) {
      error = e.toString();
    }

    // If the widget was removed from the tree while the asynchronous platform
    // message was in flight, we want to discard the reply rather than calling
    // setState to update our non-existent appearance.
    if (!mounted) return;
    if (resultList == null || resultList.isEmpty) return;

    setState(() {
      imageList = resultList;
    });
  }
}

class PostAction extends StatefulWidget {
  final String forumId;
  final String content;
  final List<Asset> imageList;

  const PostAction(this.forumId, this.content, this.imageList);

  @override
  State<StatefulWidget> createState() => PostActionState();
}

class PostActionState extends State<PostAction> with InjectApi {
  final Map<Asset, photo.Image> upload = {};
  ValueNotifier<String> _msg;
  bool _disposed = false;

  @override
  void initState() {
    super.initState();
    _disposed = false;
    _msg = ValueNotifier<String>("请稍候...");
    WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
      post();
    });
  }

  @override
  void dispose() {
    _disposed = true;
    _msg?.dispose();
    super.dispose();
  }

  void post() async {
    List<Asset> imageList = widget.imageList;
    if (imageList != null && imageList.isNotEmpty) {
      Directory tempDir = await getTemporaryDirectory();
      Directory directory = new Directory('${tempDir.path}/upload');
      if (!directory.existsSync()) {
        directory.createSync();
        print('文档初始化成功，文件保存路径为 ${directory.path}');
      }

      for (var i = 0; i < imageList.length; i++) {
        if (_disposed) break;
        Asset asset = imageList[i];
        if (upload.containsKey(asset)) continue;
        ByteData byteData = await asset.getByteData(quality: 85);
        File file = File('${directory.path}/${DateTime.now().millisecondsSinceEpoch}_$i.jpg');
        List<int> bytes = byteData.buffer.asUint8List().toList();
        print('临时文件 ${file.path} ${bytes.length}');
        file.writeAsBytesSync(bytes);
        _msg.value = "上传图片(${i + 1}/${imageList.length})...";
        var resp = await api.mediaUp4Subject(file, srcType: "image");
        photo.Image image = resp.data;
        file.delete();
        upload[asset] = image;
        // await Future.delayed(Duration(seconds: 3));
      }
    }

    _msg.value = "发布中...";
    // await Future.delayed(Duration(seconds: 3));
    if (_disposed) return;
    var data = await api.postForum(widget.forumId, widget.content, images: upload.values.map((e) => e.id).toList().join(","));
    if (data != null && data.code == 0) {
      Navigator.of(context).pop(true);
    } else {
      Navigator.of(context).pop(false);
    }
  }

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.symmetric(vertical: 16),
      child: Column(
        children: <Widget>[
          CircularProgressIndicator(),
          Padding(
            padding: const EdgeInsets.only(top: 15),
            child: ValueListenableBuilder(valueListenable: _msg, builder: (BuildContext context, String value, Widget child) => Text(value)),
          )
        ],
      ),
    );
  }
}
