import 'dart:math';

import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:sport/bean/post.dart';
import 'package:sport/bean/post_user.dart';
import 'package:sport/pages/social/gallery_photo_view.dart';
import 'package:sport/pages/social/post_detail_page.dart';
import 'package:sport/provider/social_detail_model.dart';
import 'package:sport/router/navigator_util.dart';
import 'package:sport/services/api/inject_api.dart';
import 'package:sport/utils/DateFormat.dart';
import 'package:sport/utils/toast.dart';
import 'package:sport/widgets/dialog/alert_dialog.dart';
import 'package:sport/widgets/dialog/modal_bottom_action.dart';
import 'package:sport/widgets/dialog/request_dialog.dart';
import 'package:sport/widgets/image.dart';
import 'package:sport/widgets/space.dart';

class PostWidget extends StatefulWidget {
  final SocialDetailModel model;
  final Post post;
  final bool isSelf;
  final bool jump; // 头像是否可点击
  final String keyword; // 头像是否可点击
  final bool highlight; // 头像是否可点击

  PostWidget(this.post, this.model, this.isSelf, {this.jump = true, this.highlight = false, this.keyword});

  @override
  State<StatefulWidget> createState() => _PostWidgetState();
}

class _PostWidgetState extends State<PostWidget> with InjectApi {
  @override
  Widget build(BuildContext context) {
    return widget.post.isDelete == true ? Container() : _buildPostDetailWidget(context, widget.post, widget.isSelf);
  }

  void go(Post post, bool comment) {
    Navigator.push(context, MaterialPageRoute(builder: (context) {
      return PostDetailPage(post, comment, widget.model.list);
    }));
  }

  bool isExpansion(String text) {
    TextPainter _textPainter = TextPainter(maxLines: 3, text: TextSpan(text: text, style: TextStyle(fontSize: 16.0)), textDirection: TextDirection.ltr)
      ..layout(maxWidth: MediaQuery.of(context).size.width - 24);
    if (_textPainter.didExceedMaxLines) {
      return true;
    } else {
      return false;
    }
  }

  Widget _highlightName(String content) {
    String keyword = widget.keyword;
    var list = content.split(keyword);
    List<InlineSpan> texts = [];
    for (int i = 0; i < list.length; i++) {
      texts.add(TextSpan(text: list[i]));
      if (i < list.length - 1) texts.add(TextSpan(text: keyword, style: Theme.of(context).textTheme.subtitle1.copyWith(color: Theme.of(context).accentColor)));
    }
    return RichText(
      text: TextSpan(style: Theme.of(context).textTheme.subtitle1, children: texts),
    );
  }

  Widget _highlightContent(String content, {int maxLines}) {
    String keyword = widget.keyword;
    var list = content.split(keyword);
    List<InlineSpan> texts = [];
    for (int i = 0; i < list.length; i++) {
      texts.add(TextSpan(text: list[i]));
      if (i < list.length - 1)
        texts.add(TextSpan(text: keyword, style: Theme.of(context).textTheme.subtitle1.copyWith(fontSize: 16, color: Theme.of(context).accentColor)));
    }
    return RichText(
      maxLines: maxLines,
      text: TextSpan(style: Theme.of(context).textTheme.subtitle1.copyWith(fontSize: 16), children: texts),
    );
  }

  _delete(SocialDetailModel model, Post post){
    model.list.removeWhere((element) => element.id == post.id);
    setState(() {
      post.isDelete = true;
    });
  }

  Widget _buildPostDetailWidget(BuildContext context, Post post, bool isSelf) {
    SocialDetailModel model = widget.model;
    double width = MediaQuery.of(context).size.width - 24;
    return Column(
      children: <Widget>[
        GestureDetector(
          behavior: HitTestBehavior.opaque,
          onTap: () {
            go(post, false);
            // NavigatorUtil.goSocialPostDetail(context, post)
          },
          child: Padding(
            padding: const EdgeInsets.symmetric(vertical: 12.0),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: <Widget>[
                Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: <Widget>[
                    GestureDetector(
                        onTap: widget.jump
                            ? () => NavigatorUtil.goSocialUserDetail(context, PostUser(id: post.userId, name: post.nickname, avatar: post.avatar))
                            : null,
                        child: Row(
                          children: <Widget>[
                            Space(
                              width: 12,
                            ),
                            CircleAvatar(backgroundImage: userAvatarProvider(post.avatar), radius: 18),
                            Space(
                              width: 8,
                            ),
                            Column(
                              crossAxisAlignment: CrossAxisAlignment.start,
                              children: <Widget>[
                                widget.highlight
                                    ? _highlightName(post.nickname)
                                    : Text(
                                        post.nickname,
                                        style: Theme.of(context).textTheme.subtitle1,
                                      ),
                                Space(
                                  height: 4,
                                ),
                                Text(DateFormat.formatTime(post.createTime), style: Theme.of(context).textTheme.bodyText1)
                              ],
                            ),
                          ],
                        )),
                    PopupMenuButton(
                      icon: arrowBottom(),
                      onSelected: (action) async {
                        switch (action) {
                          case "report":
                            await request(context, () async {
                              await model.api.postForumReport(subjectId: post.id).catchError((onError) {});
                              ToastUtil.show("举报成功");
                            });
                            break;
                          case "block":
                            await request(context, () async {
                              await model.api.postForumBlockObject(post.id, "subject").catchError((onError) {});
                              ToastUtil.show("屏蔽成功");
                             _delete(model, post);
                            });
                            break;
                          case "delete":
                            if (await showDialog(
                                  context: context,
                                  builder: (context) => CustomAlertDialog(title: '确定删除此条？', ok: () => Navigator.of(context).pop(true)),
                                ) ==
                                true) {
                              await request(context, () async {
                                await model.api.postDelSubject(post.id);
                                ToastUtil.show("删除成功");
                                _delete(model, post);
                              });
                            }
                            break;
                        }
                      },
                      itemBuilder: (_) => [
                        if (!isSelf)
                          PopupMenuItem(
                            value: "report",
                            child: Text("举报"),
                          ),
                        if (!isSelf)
                          PopupMenuItem(
                            value: "block",
                            child: Text("屏蔽此条"),
                          ),
                        if (isSelf)
                          PopupMenuItem(
                            value: "delete",
                            child: Text("删除此条"),
                          )
                      ],
                    )
                  ],
                ),
                Padding(
                  padding: const EdgeInsets.symmetric(horizontal: 12.0),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: <Widget>[
                      GestureDetector(
                        onLongPress: () async {
                          await showActionDialog(context, {
                            "复制文字": () {
                              Clipboard.setData(ClipboardData(text: post.content));
                              ToastUtil.show("已复制到粘贴板");
                            }
                          });
                        },
                        child: Padding(
                          padding: const EdgeInsets.symmetric(vertical: 12.0),
                          child: Column(
                            crossAxisAlignment: CrossAxisAlignment.start,
                            children: <Widget>[
                              post.isExpansion ?? false == false
                                  ? widget.highlight
                                      ? _highlightContent(post.content, maxLines: 3)
                                      : Text(post.content,
                                          maxLines: 3, overflow: TextOverflow.ellipsis, style: Theme.of(context).textTheme.subtitle1.copyWith(fontSize: 16))
                                  : widget.highlight
                                      ? _highlightContent(post.content)
                                      : Text(post.content, style: Theme.of(context).textTheme.subtitle1.copyWith(fontSize: 16)),
                              if (isExpansion(post.content))
                                GestureDetector(
                                    onTap: () {
                                      post.isExpansion = !(post.isExpansion ?? true);
                                      setState(() {});
                                    },
                                    child: Padding(
                                      padding: const EdgeInsets.only(top: 5.0),
                                      child: Text(post.isExpansion ?? false == false ? "全部" : "收起", style: Theme.of(context).textTheme.bodyText2),
                                    )),
                            ],
                          ),
                        ),
                      ),
                      if (post.images.length > 0)
                        GridView.count(
                            physics: new NeverScrollableScrollPhysics(),
                            shrinkWrap: true,
                            padding: EdgeInsets.zero,
                            childAspectRatio: post.images.length == 1 ? max(16 / 10, post.images[0].getImageAspectRatio()) : 1,
                            crossAxisSpacing: 10.0,
                            crossAxisCount: min(3, post.images.length),
                            children: post.images
                                .asMap()
                                .keys
                                .take(min(3, post.images.length))
                                .map((i) => GestureDetector(
                                    onTap: () => i < 2 ? open(context, i, post.images) : go(post, false),
                                    child: i < 2
                                        ? post.images.length == 1
                                            ? Row(
                                                mainAxisSize: MainAxisSize.min,
                                                children: <Widget>[
                                                  ClipRRect(
                                                      borderRadius: BorderRadius.circular(6),
                                                      child: Stack(
                                                        children: <Widget>[
                                                          CachedNetworkImage(
                                                            alignment: Alignment.centerLeft,
                                                            imageUrl: post.images[i].thumbnail,
                                                            fit: BoxFit.cover,
                                                            width: post.images[i].getWidth(width),
                                                          ),
                                                          if (post.images[i].isLongImage())
                                                            Positioned(
                                                              bottom: 4,
                                                              right: 4,
                                                              child: Container(
                                                                padding: EdgeInsets.symmetric(horizontal: 8, vertical: 2),
                                                                decoration: BoxDecoration(
                                                                    color: Colors.black.withOpacity(.8), borderRadius: BorderRadius.all(Radius.circular(20))),
                                                                child: Text(
                                                                  "长图",
                                                                  style: Theme.of(context).textTheme.bodyText1.copyWith(color: Colors.white),
                                                                ),
                                                              ),
                                                            )
                                                        ],
                                                      ))
                                                ],
                                              )
                                            : ClipRRect(
                                                borderRadius: BorderRadius.circular(6),
                                                child:
                                                    CachedNetworkImage(alignment: Alignment.centerLeft, imageUrl: post.images[i].thumbnail, fit: BoxFit.cover))
                                        : ClipRRect(
                                            borderRadius: BorderRadius.circular(6),
                                            child: Stack(
                                              fit: StackFit.expand,
                                              children: <Widget>[
                                                CachedNetworkImage(
                                                  imageUrl: post.images[i].thumbnail,
                                                  fit: BoxFit.cover,
                                                ),
                                                if (post.images.length - 3 > 0)
                                                  Container(
                                                    color: Color(0x80000000),
                                                    child: Center(
                                                      child: Text(
                                                        "+${post.images.length - 3}",
                                                        style: TextStyle(color: Colors.white, fontSize: 16),
                                                      ),
                                                    ),
                                                  )
                                              ],
                                            ))))
                                .toList()),
                    ],
                  ),
                ),
                Space(
                  height: 10,
                ),
                Row(
                  children: <Widget>[
                    Expanded(
                      child: GestureDetector(
                        behavior: HitTestBehavior.opaque,
                        onTap: () => ToastUtil.show("分享什么"),
                        child: Container(
                          padding: const EdgeInsets.all(5.0),
                          child: Center(
                            child: Row(
                              mainAxisSize: MainAxisSize.min,
                              children: <Widget>[
                                Image.asset("lib/assets/img/bbslist_icon_share.png"),
                                SizedBox(
                                  width: 5,
                                ),
                                Text(
                                  "转发",
                                  strutStyle: StrutStyle(forceStrutHeight: true),
                                  style: Theme.of(context).textTheme.bodyText1,
                                ),
                              ],
                            ),
                          ),
                        ),
                      ),
                    ),
                    Expanded(
                      child: GestureDetector(
                        behavior: HitTestBehavior.opaque,
                        onTap: () => go(post, true),
                        child: Container(
                          padding: const EdgeInsets.all(5.0),
                          child: Center(
                            child: Row(
                              mainAxisSize: MainAxisSize.min,
                              children: <Widget>[
                                Image.asset("lib/assets/img/bbslist_icon_reply.png"),
                                SizedBox(
                                  width: 5,
                                ),
                                Text(
                                  post.commentCount == 0 ? "回复" : "${post.commentCount}",
                                  strutStyle: StrutStyle(forceStrutHeight: true),
                                  style: Theme.of(context).textTheme.bodyText1,
                                ),
                              ],
                            ),
                          ),
                        ),
                      ),
                    ),
                    Expanded(
                      child: GestureDetector(
                        behavior: HitTestBehavior.opaque,
                        onTap: () async {
                          await (post.isLiked ? model.api.postForumUnLike(post.id, "subject_id") : model.api.postForumLike(post.id, "subject_id"));
                          setState(() {
                            post.toggleLike();
                          });
                        },
                        child: Container(
                          padding: const EdgeInsets.all(5.0),
                          child: Center(
                            child: SizedBox(
                              height: 19,
                              child: Row(
                                crossAxisAlignment: CrossAxisAlignment.end,
                                mainAxisSize: MainAxisSize.min,
                                children: <Widget>[
                                  Image.asset(
                                    "lib/assets/img/bbslist_icon_like${post.isLiked ? 'd' : ''}.png",
                                  ),
                                  SizedBox(
                                    width: 5,
                                  ),
                                  Text(
                                    post.likeCount == 0 ? "点赞" : "${post.likeCount}",
                                    strutStyle: StrutStyle(forceStrutHeight: true),
                                    style: Theme.of(context).textTheme.bodyText1,
                                  ),
                                ],
                              ),
                            ),
                          ),
                        ),
                      ),
                    ),
                  ],
                ),
              ],
            ),
          ),
        ),
        Divider(
          height: 1,
        )
      ],
    );
  }
}
