import 'package:cached_network_image/cached_network_image.dart';
import 'package:dio_http_cache/dio_http_cache.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter_easyrefresh/easy_refresh.dart';
import 'package:provider/provider.dart';
import 'package:sport/bean/post.dart';
import 'package:sport/bean/post_user.dart';
import 'package:sport/bean/user_info.dart';
import 'package:sport/pages/social/post_widget.dart';
import 'package:sport/provider/lib/provider_widget.dart';
import 'package:sport/provider/lib/view_state_lifecycle.dart';
import 'package:sport/provider/user_detail_model.dart';
import 'package:sport/provider/user_model.dart';
import 'package:sport/router/navigator_util.dart';
import 'package:sport/services/api/inject_api.dart';
import 'package:sport/services/api/resp.dart';
import 'package:sport/services/userid.dart';
import 'package:sport/utils/DateFormat.dart';
import 'package:sport/widgets/error.dart';
import 'package:sport/widgets/image.dart';
import 'package:sport/widgets/loading.dart';
import 'package:sport/widgets/misc.dart';
import 'package:sport/widgets/persistent_header.dart';
import 'package:sport/widgets/space.dart';

class UserDetailPage extends StatefulWidget {
  final PostUser user;

  UserDetailPage(this.user);

  @override
  State<StatefulWidget> createState() => _PageState();
}

class _PageState extends ViewStateLifecycle<UserDetailPage, UserDetailModel> with InjectApi, UserId {
  ScrollController _controller;
  double _expandedHeight = 0;
  int _brightness = 0;
  String _genderLabel = "他";
  static const List<String> _sort = ["时间倒序", "点赞最多", "评论最多"];
  String _sortType = _sort[0];
  ValueNotifier<UserInfo> _valueNotifierUserInfo;

  @override
  void initState() {
    super.initState();

    _valueNotifierUserInfo = ValueNotifier(UserInfo.fromJson(widget.user.toJson()));
    _controller = ScrollController()
      ..addListener(() {
        if (_controller.position.pixels >= _expandedHeight - 70) {
          if (_brightness == 0) {
            setState(() {
              _brightness = 1;
            });
          }
        } else {
          if (_brightness == 1) {
            setState(() {
              _brightness = 0;
            });
          }
        }
      });
    String me = "${Provider.of<UserModel>(context, listen: false).user.id}";
    print(me == widget.user.id);
    api.getUserInfo(widget.user.id, options: me == widget.user.id ? null : buildCacheOptions(Duration(hours: 1))).then((value) {
      _valueNotifierUserInfo.value = value.data;
      _genderLabel = value.data.gender == 1 ? "他" : "她";
    });
  }

  @override
  void dispose() {
    super.dispose();
    _controller?.dispose();
    _valueNotifierUserInfo?.dispose();
  }

  @override
  Widget build(BuildContext context) {
    _expandedHeight = MediaQuery.of(context).size.width * 552.0 / 1125 - MediaQuery.of(context).padding.top;
//    print(MediaQuery.of(context));
    return Scaffold(
      backgroundColor: Colors.white,
      body: ProviderWidget<UserDetailModel>(
          model: model,
          onModelReady: (model) => model.initData(),
          builder: (_, model, __) {
            return EasyRefresh.custom(
              controller: model.refreshController,
              enableControlFinishRefresh: true,
              enableControlFinishLoad: true,
              scrollController: _controller,
              onRefresh: () => model.refresh(),
              onLoad: model.isIdle ? () => model.loadMore() : null,
              header: buildClassicalHeader(
                extent: 100.0,
                triggerDistance: 120.0,
              ),
              footer: buildClassicalFooter(),
              slivers: <Widget>[
                SliverAppBar(
                  expandedHeight: _expandedHeight,
                  pinned: true,
                  elevation: 0,
                  forceElevated: true,
                  iconTheme: IconThemeData(color: _brightness == 0 ? Colors.white : Colors.black),
                  brightness: _brightness == 0 ? Brightness.dark : Brightness.light,
                  backgroundColor: _brightness == 0 ? Colors.transparent : Colors.white,
                  leading: IconButton(
                    icon: Image.asset("lib/assets/img/topbar_return${_brightness == 0?"_white":""}.png"),
                    onPressed: () {
                      Navigator.of(context).pop();
                    },
                  ),
                  flexibleSpace: FlexibleSpaceBar(
                    collapseMode: CollapseMode.pin,
                    stretchModes: [],
                    background: Stack(
                      alignment: Alignment.bottomCenter,
                      overflow: Overflow.visible,
                      children: <Widget>[
                        Image.asset(
                          "lib/assets/img/bg_userprofile_bbs.jpg",
                          fit: BoxFit.fitWidth,
                        ),
                        Positioned(
                          left: 0,
                          right: 0,
                          bottom: -1,
                          child: Container(
                            height: 10,
                            decoration: BoxDecoration(borderRadius: BorderRadius.vertical(top: Radius.circular(10)), color: Colors.white),
                          ),
                        ),
                        ValueListenableBuilder(
                          valueListenable: _valueNotifierUserInfo,
                          builder: (BuildContext context, UserInfo user, Widget child) {
                            if (user == null) {
                              return Container();
                            }
                            return Padding(
                              padding: const EdgeInsets.symmetric(vertical: 24, horizontal: 12.0),
                              child: Container(
                                height: 80,
                                child: Row(
                                  children: <Widget>[
                                    CircleAvatar(
                                      backgroundImage: userAvatarProvider(user.avatar),
                                      radius: 30,
                                    ),
                                    Space(
                                      width: 12,
                                    ),
                                    Expanded(
                                      child: Column(
                                        mainAxisAlignment: MainAxisAlignment.center,
                                        crossAxisAlignment: CrossAxisAlignment.start,
                                        children: <Widget>[
                                          Text(user.name, style: TextStyle(color: Colors.white, fontSize: 18)),
                                          Space(
                                            height: 7,
                                          ),
                                          Row(
                                            children: <Widget>[
                                              Image.asset("lib/assets/img/mine_icon_${user.gender == 1 ? "manl" : "girl"}_white.png"),
                                              Space(
                                                width: 4,
                                              ),
                                              Text(
                                                user.gender == 1 ? "男" : "女",
                                                style: TextStyle(color: Colors.white, fontSize: 14),
                                              ),
                                              Space(
                                                width: 10,
                                              ),
                                              Text(
                                                user.age != null ? "${user.age}岁" : "",
                                                style: TextStyle(color: Colors.white, fontSize: 14),
                                              ),
                                              if ((user.districtId ?? 0) != 0)
                                                Padding(
                                                  padding: const EdgeInsets.only(left: 10.0),
                                                  child: Text(
                                                    "${user.province}${user.province == user.city ? '' : user.city}",
                                                    style: TextStyle(color: Colors.white, fontSize: 14),
                                                  ),
                                                )
                                            ],
                                          )
                                        ],
                                      ),
                                    )
                                  ],
                                ),
                              ),
                            );
                          },
                        )
                      ],
                    ),
                  ),
                ),
                if (model.isBusy) SliverToBoxAdapter(child: RequestLoadingWidget()),
                if (model.isEmpty)
                  SliverToBoxAdapter(
                      child: RequestErrorWidget(
                    null,
                    msg: "暂无帖子~",
                    assets: RequestErrorWidget.ASSETS_NO_MOTION,
                  )),
                if (model.isIdle)
                  SliverToBoxAdapter(
                    child: Container(
                      color: Colors.white,
                      child: ValueListenableBuilder<UserInfo>(
                          valueListenable: _valueNotifierUserInfo,
                          builder: (_, data, ___) => (data == null || data.achievements == null || data.achievements.isEmpty)
                              ? Container()
                              : Column(
                                  children: <Widget>[
                                    Padding(
                                      padding: const EdgeInsets.all(12.0),
                                      child: Stack(
                                        alignment: Alignment.center,
                                        children: <Widget>[
                                          Image.asset(
                                            "lib/assets/img/mine_image_achievement.png",
                                            fit: BoxFit.fitWidth,
                                            width: 240,
                                          ),
                                          Text(
                                            "${widget.user.id == Provider.of<UserModel>(context).user.id.toString() ? '我' : '$_genderLabel'}的成就",
                                            style: Theme.of(context).textTheme.subtitle1,
                                          )
                                        ],
                                      ),
                                    ),
                                    Padding(
                                      padding: const EdgeInsets.all(12.0),
                                      child: achievementGroupWidget(data?.achievements),
                                    ),
                                    Divider()
                                  ],
                                )),
                    ),
                  ),
                if (model.isIdle)
                  SliverPersistentHeader(
                    delegate: PersistentHeader(
                        min: 50,
                        max: 50,
                        child: Container(
                          color: Colors.white,
                          padding: EdgeInsets.only(bottom: 5),
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.spaceBetween,
                            children: <Widget>[
                              Padding(
                                padding: const EdgeInsets.all(12.0),
                                child: Text(
                                  "$_genderLabel的帖子",
                                  style: Theme.of(context).textTheme.headline3,
                                ),
                              ),
                              PopupMenuButton(
                                onSelected: (v) {
                                  setState(() {
                                    _sortType = v;
                                    model.sortBy(_sortType);
                                  });
                                },
                                itemBuilder: (BuildContext context) {
                                  return _sort
                                      .map((e) => PopupMenuItem(
                                            child: Text(e),
                                            value: e,
                                          ))
                                      .toList();
                                },
                                child: Row(
                                  mainAxisSize: MainAxisSize.min,
                                  children: <Widget>[
                                    Text(
                                      _sortType,
                                      style: Theme.of(context).textTheme.subtitle1,
                                    ),
                                    Padding(
                                      padding: const EdgeInsets.fromLTRB(6, 6, 12.0, 6),
                                      child: arrowBottom(),
                                    )
                                  ],
                                ),
                              ),
                            ],
                          ),
                        )),
                    pinned: true,
                  ),
                if (model.isIdle)
                  SliverList(
                    delegate: SliverChildBuilderDelegate(
                      (context, index) {
                        Post post = model.list[index];
                        return PostWidget(post, model, selfId == post.userId, jump: false,);
                      },
                      childCount: model.list.length,
                    ),
                  ),
              ],
            );
          }),
    );
  }

  @override
  createModel() => UserDetailModel(widget.user.id.toString());
}
