import 'dart:async';
import 'dart:io';

import 'package:android_intent/android_intent.dart';
import 'package:flutter_blue/flutter_blue.dart';
import 'package:sport/bean/BleDevice.dart';
import 'package:sport/provider/lib/view_state_model.dart';

class BleModel extends ViewStateModel {
  FlutterBlue _flutterBlue = FlutterBlue.instance;
  List<ScanResult> _results = [];
  List<BleDevice> devices = [];
  bool inOn;
  bool isScanning = false;
  StreamSubscription _subscription;

  Future<bool> checkBluetooth() {
    return _flutterBlue.isAvailable;
  }

  Future<bool> checkBluetoothAdapter() {
    return _flutterBlue.isOn;
  }

  requestSettings() async {
    if (Platform.isAndroid) {
      String action = "android.bluetooth.adapter.action.REQUEST_ENABLE";
      final AndroidIntent intent = AndroidIntent(action: action);
      await intent.launch();
    }
  }

  startScan() async {
    inOn = await _flutterBlue.isOn;
    if (!inOn) {
      return;
    }
    print('start devices found! devices: ${devices.length}');
    isScanning = true;
    _flutterBlue.startScan(timeout: Duration(seconds: 5)).whenComplete(() => isScanning = false).catchError((onError) => setError(onError, null));

    _subscription = _flutterBlue.scanResults.listen((results) {
      for (ScanResult r in results) {
        print('${r.device.name} found! rssi: ${r.rssi}');
        if (r.device.name.isNotEmpty && _results.indexOf(r) < 0) {
          _results.add(r);
          devices.add(BleDevice(result: r));
        }
      }
      print('devices found! devices: ${devices.length} $this');
      notifyListeners();
    });
  }

  @override
  void dispose() {
    _flutterBlue.stopScan().catchError((onError) => setError(onError, null));
    _subscription?.cancel();
    super.dispose();
  }

  connect(int index) async {
    BleDevice device = BleDevice(result: devices[index].result, state: 1);
    devices[index] = device;
    notifyListeners();
    BluetoothDevice bluetoothDevice = device.result.device;
    await bluetoothDevice.connect();
    List<BluetoothService> services = await bluetoothDevice.discoverServices();
    services.forEach((service) {
      // do something with service
    });
//    await characteristic.setNotifyValue(true);
//    characteristic.value.listen((value) {
//      // do something with new value
//    });
  }
}
