import 'dart:io';

import 'package:dio/dio.dart' hide Headers;
import 'package:retrofit/retrofit.dart';
import 'package:sport/bean/UpdateInfo.dart';
import 'package:sport/bean/comment.dart';
import 'package:sport/bean/comment_post.dart';
import 'package:sport/bean/feedback_category.dart';
import 'package:sport/bean/forum.dart';
import 'package:sport/bean/game.dart';
import 'package:sport/bean/image.dart';
import 'package:sport/bean/notice.dart';
import 'package:sport/bean/post.dart';
import 'package:sport/bean/rank_game_info.dart';
import 'package:sport/bean/rank_info.dart';
import 'package:sport/bean/sport_detail.dart';
import 'package:sport/bean/sport_index.dart';
import 'package:sport/bean/sport_target.dart';
import 'package:sport/bean/sport_target_today.dart';
import 'package:sport/bean/user_info.dart';
import 'package:sport/services/api/resp.dart';

part 'rest_client.g.dart';

@RestApi()
abstract class RestClient {
  factory RestClient(Dio dio, {String baseUrl}) = _RestClient;

  @GET("/app/checkUpdate")
  Future<RespData<UpdateInfo>> checkUpdate(@Query("version") String version);

  @GET("/user/info")
  Future<RespData<UserInfo>> getUserInfo(@Query("id") String uid, {@DioOptions() Options options});

  // ********************** 运动首页相关 *****************************
  @GET("/sport/index")
  Future<RespData<SportIndex>> getSportIndex();

  @GET("/sport/detail")
  Future<RespData<SportDetail>> getSportDetail();

  @POST("/sportRecord/addGame")
  Future<RespList<SportTarget>> postAddGame(
      @Query("game_id") int gameId, @Query("score") double score, @Query("duration") int duration, @Query("consume") int consume, @Query("screen") int screen);

  @GET("/sportTarget/all")
  Future<RespList<SportTarget>> getSportTargetAll();

  @GET("/sportTarget/set")
  Future<RespData<SportTargetToday>> setSportTarget(@Query("id") int id);

  @GET("/sportTarget/setCustom")
  Future<RespData<SportTargetToday>> setSportTargetCustom(@Query("duration") int duration, @Query("consume") int consume);

  @GET("/sportTarget/del")
  Future<Resp> delSportTarget();

  @GET("/sportTarget/my")
  Future<RespData<SportTargetToday>> getSportTargetMy();

  @GET("/sportRecord/listOneDay")
  Future<RespData<SportDetailSimple>> getSportRecordListOneDay(@Query("date") String date);

  @GET("/sportRecord/listByDay")
  Future<RespData<SportDetailSimple>> getSportRecordListByDay(@Query("begin") String begin, @Query("end") String end);

  @GET("/sportRecord/listByMonth")
  Future<RespData<SportDetailSimple>> getSportRecordListByMonth(@Query("year") int year);

  @GET("/sportRecord/listByYear")
  Future<RespData<SportDetailSimple>> getSportRecordListByYear();

  @GET("/sportRecord/list")
  Future<RespPage<RecordsTodaySum>> getSportRecordList(@Query("game_id") int gameId, {@Query("page") int page});

  @GET("/rank/game")
  Future<RespData<RankInfo>> getRankInfo(@Query("game_id") String gameId,
      {@Query("province_id") int provinceId, @Query("city_id") int cityId, @Query("district_id") int districtId});

  // ********************** 社区相关 *****************************
  @GET("/forum/index")
  Future<RespList<Forum>> getForumIndex();

  @GET("/forum/list")
  Future<RespPage<Post>> getPostList({@Query("forumId") String forumId, @Query("p") int page, @Query("isGood") int isGood, @Query("kw") String kw});

  @GET("/forum/listByHot")
  Future<RespPage<Post>> getPostListByHot(@Query("forumId") String forumId, {@Query("p") int page, @DioOptions() Options options});

  @GET("/forum/listByUser")
  Future<RespPage<Post>> getPostListByUser(@Query("uid") String uid, {@Query("forumId") String forumId, @Query("p") int page, @Query("sortBy") String sortBy});

  @GET("/forum/subjectDetail")
  Future<RespData<Post>> getPostDetail(@Query("subjectId") String subjectId);

  @GET("/forum/comments")
  Future<RespPage<Comment>> getPostComments(@Query("subjectId") String subjectId, {@Query("p") int page, @Query("sortBy") String sortBy});

  @GET("/forum/subComments")
  Future<RespPage<Comment>> getPostCommentSubs(@Query("parentCommentId") String parentCommentId, {@Query("p") int page, @Query("sortBy") String sortBy});

  @GET("/forum/commentsFromId")
  Future<RespList<Comment>> getPostCommentsFromId(@Query("commentId") String commentId, {@Query("limit") int limit, @Query("sortBy") String sortBy});

  @GET("/forum/hotSearchWords")
  Future<RespList<String>> getSearchWords();

  @POST("/forum/like")
  Future<RespData<String>> postForumLike(@Query("objectId") String objectId, @Query("fieldName") String fieldName);

  @POST("/forum/unlike")
  Future<RespData<String>> postForumUnLike(@Query("objectId") String objectId, @Query("fieldName") String fieldName);

  @POST("/forum/delSubject")
  Future<RespData<String>> postDelSubject(@Query("subjectId") String subjectId);

  @POST("/forum/delComment")
  Future<RespData<String>> postDelComment(@Query("commentId") String commentId);

  @POST("/forum/report")
  Future<RespData<String>> postForumReport({@Query("subjectId") String subjectId, @Query("commentId") String commentId, @Query("content") String content});

  @POST("/forum/blockUser")
  Future<RespData<String>> postForumBlockUser(@Query("uid") String uid);

  @POST("/forum/blockObject")
  Future<RespData<String>> postForumBlockObject(@Query("objectId") String objectId,@Query("for") String forName);

  @POST("/forum/postComment")
  Future<RespData<CommentPost>> postForumComment(@Query("subjectId") String subjectId, @Query("content") String content,
      {@Query("parentCommentId") String parentCommentId, @Query("toCommentId") String toCommentId});

  @POST("/forum/post")
  Future<RespData<Map<String, dynamic>>> postForum(@Query("forumId") String forumId, @Query("content") String content, {@Query("images") String images, CancelToken cancelToken});

  // ********************** 公告相关 *****************************
  @GET("/inform/list")
  Future<RespPage<Notice>> getInformList({@Query("p") int page});

  @POST("/inform/read")
  Future<RespData> postInformRead(@Query("informId") int informId);

  @GET("/notice/list?markread=currpage")
  Future<RespPage<Notice>> getNoticeList(@Query("type") String type, {@Query("p") int page, @Query("read") String read});

  @GET("/notice/count")
  Future<RespData<int>> getNoticeCount(@Query("type") String type,  @Query("read") String read);

  @POST("/notice/read")
  Future<RespData<Notice>> postNoticeRead(@Query("noticeId") int noticeId);

  // ********************** 反馈相关 *****************************

  @GET("/feedback/types")
  Future<RespList<FeedbackCategory>> getFeedbackTypes();

  @GET("/feedback/myChats")
  Future<RespList<FeedbackCategory>> getFeedbackChat({@Query("groupId") int groupId, @Query("typeId") int typeId});

  @POST("/feedback/post")
  Future<RespData<String>> postFeedback(@Query("typeId") int typeId, @Query("content") String content, {@Query("images") String images, @Query("extra") String extra});

  @POST("/media/Up4Feedback")
  @Headers(<String, dynamic>{
    "content-type": "multipart/form-data",
  })
  Future<RespData<String>> mediaUp4Feedback(@Part(name: "media") File file);

  @POST("/media/Up4Subject")
  @Headers(<String, dynamic>{
    "content-type": "multipart/form-data",
  })
  Future<RespData<Image>> mediaUp4Subject(@Part(name: "media") File file, {@Query("srcType") String srcType, @Query("subjectId") String subjectId, CancelToken cancelToken});

  // ********************** 游戏相关 *********************************
  @GET("/game/all")
  Future<RespList<GameInfoData>> getGameAll();

  @GET("/game/all")
  Future getGameById(@Query("id") int id);

  @GET("/rank/all")
  Future<RespList<RankInfoData>> getRankInfoAll();

  @GET("/rank/game")
  Future<RespData<RankGameInfoData>> getRankGameInfo(@Query("game_id") String id,
      {@Query("province_id") int provinceId, @Query("city_id") int cityId, @Query("district_id") int districtId});

  @GET("/rank/sport")
  Future<RespData<RankGameInfoData>> getRankSportInfo(@Query("rank_id") String id,
      {@Query("province_id") int provinceId, @Query("city_id") int cityId, @Query("district_id") int districtId});
}
