import 'dart:math';

import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/material.dart';
import 'package:flutter_easyrefresh/easy_refresh.dart';
import 'package:flutter_staggered_grid_view/flutter_staggered_grid_view.dart';
import 'package:sport/bean/post_user.dart';
import 'package:sport/bean/user.dart';
import 'package:sport/constant/ui.dart';
import 'package:sport/router/navigator_util.dart';
import 'package:sport/utils/DateFormat.dart';
import 'package:sport/widgets/image.dart';
import 'package:sport/widgets/refresh_footer.dart';
import 'package:sport/widgets/space.dart';

Widget buildSocialUserWidget(BuildContext context, PostUser user, int createTime) {
  return user == null
      ? Container()
      : GestureDetector(
          onTap: () => NavigatorUtil.goSocialUserDetail(context, user),
          child: Row(
            children: <Widget>[
              Space(
                width: 12,
              ),
              CircleAvatar(backgroundImage: userAvatarProvider(user?.avatar), radius: 18),
              Space(
                width: 8,
              ),
              Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: <Widget>[
                  Text(
                    user?.name,
                    style: Theme.of(context).textTheme.subtitle1.copyWith(fontWeight: FontWeight.bold),
                  ),
                  Space(
                    height: 4,
                  ),
                  Text(DateFormat.formatTime(createTime), style: Theme.of(context).textTheme.bodyText1)
                ],
              ),
            ],
          ));
}

Widget buildLabelWidget(BuildContext context, String title) {
  return Container(
      padding: EdgeInsets.fromLTRB(ui_padding, 10.0, ui_padding, 10.0),
      child: Text(
        title,
        style: Theme.of(context).textTheme.headline1,
      ));
}

Widget gameTag(BuildContext context, String name) {
  return Container(
    decoration: BoxDecoration(
      borderRadius: BorderRadius.all(Radius.circular(50)),
      border: Border.all(
        color: Theme.of(context).accentColor,
        width: 1,
      ),
    ),
    padding: EdgeInsets.symmetric(vertical: 0, horizontal: 8),
    child: Text(
      name,
      style: Theme.of(context).textTheme.bodyText1.copyWith(color: Theme.of(context).accentColor),
    ),
  );
}

Widget achievementGroupWidget(List<Achievement> achievements) {
  return achievements == null || achievements.isEmpty
      ? Container(height: 90, child: Padding(padding: const EdgeInsets.all(24.0), child: Text("还未获得成就")))
      : StaggeredGridView.countBuilder(
          padding: EdgeInsets.zero,
          shrinkWrap: true,
          physics: NeverScrollableScrollPhysics(),
          crossAxisCount: 4,
          itemCount: min(4, achievements.length),
          itemBuilder: (BuildContext context, int index) => achievementWidget(context, achievements[index]),
          mainAxisSpacing: 12.0,
          crossAxisSpacing: 12.0,
          staggeredTileBuilder: (int index) => StaggeredTile.fit(1),
        );
}

Widget achievementWidget(BuildContext context, Achievement item, {bool replace = false}) => InkWell(
    onTap: () => NavigatorUtil.goAchievementDetails(context, id: item.id, replace: replace),
    child: Column(
      children: <Widget>[
        CircleAvatar(backgroundImage: CachedNetworkImageProvider(item.logo == null ? "" : item.logo), radius: 33.0),
        Space(
          height: 5,
        ),
        Text(
          item.name,
          style: Theme.of(context).textTheme.subtitle2,
        )
      ],
    ));

Widget sportBeEquivalentTo(BuildContext context, int consume, {bool highlight = false}) => Row(
      mainAxisAlignment: MainAxisAlignment.spaceAround,
      children: <Widget>[
        Column(
          children: <Widget>[
            Image.asset("lib/assets/img/gamedetail_image_walk.png"),
            Space(
              height: 8,
            ),
            RichText(
              text: TextSpan(children: <InlineSpan>[
                TextSpan(text: '步行', style: Theme.of(context).textTheme.bodyText1),
                TextSpan(
                    text: '${(consume / 3 * 90).floor()}',
                    style: highlight
                        ? Theme.of(context).textTheme.bodyText1.copyWith(color: Theme.of(context).accentColor)
                        : Theme.of(context).textTheme.bodyText1),
                TextSpan(text: '步', style: Theme.of(context).textTheme.bodyText1),
              ]),
            )
          ],
        ),
        Column(
          children: <Widget>[
            Image.asset("lib/assets/img/gamedetail_image_run.png"),
            Space(
              height: 8,
            ),
            RichText(
              text: TextSpan(children: <InlineSpan>[
                TextSpan(text: '跑步', style: Theme.of(context).textTheme.bodyText1),
                TextSpan(
                    text: '${(consume / 60 / 1.036).toStringAsFixed(1)}',
                    style: highlight
                        ? Theme.of(context).textTheme.bodyText1.copyWith(color: Theme.of(context).accentColor)
                        : Theme.of(context).textTheme.bodyText1),
                TextSpan(text: '公里', style: Theme.of(context).textTheme.bodyText1),
              ]),
            )
          ],
        ),
        Column(
          children: <Widget>[
            Image.asset("lib/assets/img/gamedetail_image_riding.png"),
            Space(
              height: 8,
            ),
            RichText(
              text: TextSpan(children: <InlineSpan>[
                TextSpan(text: '单车', style: Theme.of(context).textTheme.bodyText1),
                TextSpan(
                    text: '${(consume / 60 / 0.6142).toStringAsFixed(1)}',
                    style: highlight
                        ? Theme.of(context).textTheme.bodyText1.copyWith(color: Theme.of(context).accentColor)
                        : Theme.of(context).textTheme.bodyText1),
                TextSpan(text: '公里', style: Theme.of(context).textTheme.bodyText1),
              ]),
            )
          ],
        ),
      ],
    );

const REFRESH_INFO_COLOR = Color(0xff999999);

ClassicalHeader buildClassicalHeader(
    {double extent = 60.0,
    double triggerDistance = 70.0,
    Color infoColor = REFRESH_INFO_COLOR,
    Color textColor = Colors.black,
    Color bgColor = Colors.transparent}) {
  return ClassicalHeader(
    extent: extent,
    triggerDistance: triggerDistance,
    showInfo: false,
    refreshText: '下拉刷新',
    refreshFailedText: '刷新失败',
    refreshedText: '刷新完成',
    refreshingText: '正在刷新...',
    refreshReadyText: '释放刷新',
    infoColor: Color(0xff999999),
    bgColor: bgColor,
    textColor: textColor,
  );
}

Footer buildClassicalFooter() {
  return CustomClassicalFooter(
      showInfo: false,
      loadedText: '加载完成',
      loadReadyText: '释放加载',
      loadFailedText: '加载失败',
      loadText: '拉动加载',
      loadingText: '正在加载...',
      noMoreText: "没有更多了~",
      infoColor: Color(0xff999999));
}
